/*	libgemwidget - Gtk+ Embedded Widget library
 *
 *	Authors: YE Nan <nan.ye@orange-ftgourp.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#ifndef __GEM_HASH_VIEW_H
#define __GEM_HASH_VIEW_H

#include <gtk/gtk.h>

#include "gemscrolledwindow.h"
#include "gemhashviewcell.h"

#ifdef __cplusplus
extern "C" {
#endif

#define	GEM_TYPE_HASH_VIEW							(gem_hash_view_get_type ())
#define	GEM_HASH_VIEW(obj)							(G_TYPE_CHECK_INSTANCE_CAST ((obj), GEM_TYPE_HASH_VIEW, GemHashView))
#define	GEM_HASH_VIEW_CLASS(klass)			(G_TYPE_CHECK_CLASS_CAST ((klass), GEM_TYPE_HASH_VIEW, GemHashViewClass))
#define	GEM_IS_HASH_VIEW(obj)						(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEM_TYPE_HASH_VIEW))
#define GEM_IS_HASH_VIEW_CLASS(klass)		(G_TYPE_CHECK_CLASS_TYPE ((klass), GEM_TYPE_HASH_VIEW))
#define GEM_HASH_VIEW_GET_CLASS(obj)		(G_TYPE_INSTANCE_GET_CLASS ((obj), GEM_TYPE_HASH_VIEW, GemHashViewClass))

typedef struct _GemHashView					GemHashView;
typedef struct _GemHashViewClass		GemHashViewClass;

/*typedef enum
  {
    AB_IV_UID = 0,
    AB_IV_FIRST_NAME,
    AB_IV_MID_NAME,
    AB_IV_LAST_NAME,
    AB_IV_MOBILE_NUM,
    AB_IV_HOME_NUM,
    AB_IV_BUSI_NUM,
    AB_IV_FAXNUM,
    AB_IV_COMPANY,
    AB_IV_DEPART,
    AB_IV_WEBSITE,
    AB_IV_EMAIL,
    AB_IV_PICTURE,
    AB_IV_RINGTONE,
    AB_IV_HADDR_STREET,
    AB_IV_HADDR_CITY,
    AB_IV_HADDR_STATE,
    AB_IV_HADDR_ZIP,
    AB_IV_HADDR_COUNTRY,
    AB_IV_BADDR_STREET,
    AB_IV_BADDR_CITY,
    AB_IV_BADDR_STATE,
    AB_IV_BADDR_ZIP,
    AB_IV_BADDR_COUNTRY,
    NUM_AB_IV_CELLS,
  } GemHashViewID;*/




typedef	enum {
	GEM_HV_STATUS_SAVE,
	GEM_HV_STATUS_CANCEL,
} GemHashViewStatus;

struct _GemHashView
{
	GemScrolledWindow					 window;
	
	GtkWidget									*layout;
	GHashTable								*items;

	gboolean									 editable;
	GemHashViewStatus					 status;
	gint											 focused_item;
};

struct _GemHashViewClass
{
	GemScrolledWindowClass		 parent_class;
};

GType										gem_hash_view_get_type					(void)	G_GNUC_CONST;
GtkWidget*							gem_hash_view_new								(void);

void										gem_hash_view_set_item					(GemHashView						*view,
																												 gint										 id,
																												 GemHashViewCell				*cell);
GemHashViewCell *				gem_hash_view_get_item					(GemHashView						*view,
																												 gint										 id);
G_CONST_RETURN GArray *	gem_hash_view_get_ids						(GemHashView						*view);

void										gem_hash_view_set_value					(GemHashView						*view,
																												 gint										 id,
																												 const gchar						*value);
G_CONST_RETURN gchar *	gem_hash_view_get_value					(GemHashView						*view,
																												 gint										 id);
																												 
void										gem_hash_view_set_focus_item		(GemHashView						*view,
																												 gint										 id);
gint										gem_hash_view_get_focus_item		(GemHashView						*view);

void										gem_hash_view_set_editable			(GemHashView						*view,
																												 gboolean								 editable);
gboolean								gem_hash_view_get_editable			(GemHashView						*view);

void										gem_hash_view_clear							(GemHashView						*view);
#ifdef __cplusplus
}
#endif

#endif /*__GEM_HASH_VIEW_H*/

/*
vi:ts=2:nowrap:ai:expandtab 
*/
