/*	libgemwidget - Gtk+ Embedded Widget library
 *
 *	Authors: YE Nan <nan.ye@orange-ftgourp.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <stdio.h>
#include <stdlib.h>

#include "gemenv.h"

#define	DEFAULT_GEM_PATH PREFIX

static gchar * gem_env_path				= NULL;
static gchar * gem_env_pic_path		= NULL;

G_CONST_RETURN	gchar *
gem_env_get_path (void)
{
	gchar * env = NULL;
	
	if (gem_env_path)
	{
		return gem_env_path;
	}	
		
	env = getenv("GEM_PATH");
	if (env)
	{
		gem_env_path = g_strdup(env);
	}
	else
	{
		gem_env_path = g_strdup(DEFAULT_GEM_PATH);
	}
	
	return gem_env_path;
}

G_CONST_RETURN	gchar *
gem_env_get_resource_path (void)
{
	gchar * env = NULL;
	
	if (gem_env_pic_path)
	{
		return gem_env_pic_path;
	}	
		
	env = getenv("GEM_PATH");
	if (env)
	{
		gem_env_pic_path = g_build_filename(env,
																				"share/gem",
																				NULL);
	}
	else
	{
		gem_env_pic_path = g_build_filename(DEFAULT_GEM_PATH,
																				"share/gem",
																				NULL);
	}
	
	return gem_env_pic_path;
}

gboolean
gem_env_init (void)
{
	gem_env_path			= NULL;
	gem_env_pic_path	= NULL;
	
	return TRUE;
}

void
gem_env_destroy (void)
{
	if (gem_env_path)
	{
		g_free(gem_env_path);
	}

	if (gem_env_pic_path)
	{
		g_free(gem_env_pic_path);
	}

	return;
}

/*
vi:ts=2:nowrap:ai:expandtab 
*/
