/*	libgemwidget - Gtk+ Embedded Widget library
 *
 *	Authors: YE Nan <nan.ye@orange-ftgourp.com> 
 *	         CHEN YaJing <chenyajing1226@gmail.com>
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <libintl.h>
#include <ctype.h>
#include <string.h>
#include <time.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <libgnomevfs/gnome-vfs.h>
#include "gemfilechooser.h"
#include "gemresource.h"
#include "gemenv.h"
#include <gdk-pixbuf/gdk-pixbuf.h>
#ifndef DBGING
#define DBGING  1
#define DBG(x...) do { if(DBGING) { \
                                   g_print("%s(%d):", __FUNCTION__, __LINE__); \
                                   g_print(x);    \
                                   g_print("\n"); \
                              } \
                              }while(0)
#endif
#define _(x) gettext(x)
static gchar *prefix =  PREFIX ;
extern gchar *virtual_root =  VIRTUAL_ROOT;
static gchar * gem_resource_path = NULL;
static GHashTable *loaded_icons;
static GnomeVFSMonitorHandle *dir_monitor = NULL;
static gboolean refresh_scheduled = FALSE;
static void 
free_a_hash_table_entry(gpointer key, gpointer value, gpointer user_data);

static void
free_a_hash_table_entry(gpointer key, gpointer value, gpointer user_data)
{
  g_free(key);
  gdk_pixbuf_unref((GdkPixbuf *)value);
  return;
}
//gchar *sdcard_path =   SD_PATH;
typedef	enum {
	GEM_FC_COL_CHECK,
	GEM_FC_COL_PIC,
	GEM_FC_COL_NAME,
	GEM_FC_COL_IS_FOLDER,
	GEM_FC_COL_VFS_FILE_INFO,
	GEM_FC_COL_CHECK_BOX,
	GEM_FC_NUMS_COLS,
} GemFileChooserCols;

/* GObject, GtkObject methods
 */
static void gem_file_chooser_class_init			(GemFileChooserClass	*klass);
static void gem_file_chooser_init						(GemFileChooser				*window);
static void	gem_file_chooser_dispose				(GObject							*object);
static void	gem_file_chooser_finalize				(GObject							*object);
static gchar *find_mime_icon(GnomeVFSFileInfo*,gchar *);
static gchar *find_icon_path (gchar *);
static GdkPixbuf *get_pixbuf (const gchar* );
/* Default signal handlers
 */
static void gem_file_chooser_file_selected	(GemFileChooser				*filechooser);

static void
gem_file_chooser_up_to_vroot(GemFileChooser	*filechooser);

static void
gem_file_chooser_enter_to_sub_dir(GemFileChooser	*filechooser);

static GemTreeView * parent_class = NULL;

static gchar * 
find_mime_icon( GnomeVFSFileInfo * vfs_file_info , gchar *absolute_path)
{
  gchar *mime_icon = NULL;
  gchar *path= g_strdup(absolute_path);
  if (vfs_file_info->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
  {   
    mime_icon = g_strdup_printf ("%s/%s/%s",gem_resource_path,"pixmaps", "folder.png");
  }
  else 
    if( vfs_file_info->type == GNOME_VFS_FILE_TYPE_REGULAR||vfs_file_info->type == GNOME_VFS_FILE_TYPE_UNKNOWN )
    {  
      vfs_file_info->mime_type = gnome_vfs_get_mime_type (path);
      if (vfs_file_info->mime_type)
      { 
        mime_icon = find_icon_path (vfs_file_info->mime_type);
      }
      else
      {  
        mime_icon = g_strdup_printf ("%s/%s/%s",gem_resource_path,"pximaps","file.png");
      } 
    }
  g_free(path);
  return mime_icon;
}

static gchar *
find_icon_path (gchar *mime_type)
{
  struct stat s;
  gchar *mime_icon;
  gchar *mime_path, *p;
  mime_icon = g_strdup(gnome_vfs_mime_get_icon(mime_type));
  if (mime_icon)
  {
    if (mime_icon[0] == '/')
    {
      if (stat (mime_icon, &s) == 0)
         return mime_icon;
    }
    mime_path = g_strdup_printf ("%s/%s/%s",gem_resource_path,"pixmaps",mime_icon);
    g_free(mime_icon);
    
    if (stat (mime_path, &s) == 0)
      return mime_path;
    g_free(mime_path);
    mime_path = g_strdup_printf ("%s/%s/%s",gem_resource_path, "pixmaps",mime_icon);
    if (stat (mime_path, &s) == 0)
      return mime_path;
  }
  //not find in the mime database ,mime_icon should be mime_type
  mime_icon = g_strdup (mime_type);
   
  while ((p = strchr(mime_icon, '/')) != NULL)
    *p = '-';
  mime_path = g_strdup_printf ("%s/%s/gnome-%s.png",gem_resource_path,"pixmaps",mime_icon);
  if (stat (mime_path, &s) == 0)
  {
    g_free(mime_icon);
    return mime_path;
  }
  
  g_free(mime_path);
  mime_path = g_strdup_printf ("%s/%s/gnome-%s.png",gem_resource_path , "pixmaps",mime_icon);
  if (stat (mime_path, &s) == 0)
  {
    g_free(mime_icon);
    return mime_path;
  }
  g_free(mime_icon);
  g_free(mime_path);
  return g_strdup_printf ("%s/%s/file.png",gem_resource_path,"pixmaps");
}

static GdkPixbuf *
get_pixbuf (const gchar * filename)
{
  GdkPixbuf *pixbuf=NULL;
  pixbuf = g_hash_table_lookup (loaded_icons, (gconstpointer) filename);
  if (pixbuf)
  {
    g_free((gchar *)filename);
    return pixbuf;
  }
  else
  {
    pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
    g_hash_table_insert (loaded_icons, (gpointer) filename,  (gpointer) pixbuf);
    return pixbuf;
  }
}
enum {
  FILE_SELECTED,
  UP_TO_VROOT,
  ENTER_TO_SUB_DIR,
  LAST_SIGNAL
};

static guint file_chooser_signals[LAST_SIGNAL];

GType
gem_file_chooser_get_type (void)
{
	static GType file_chooser_type = 0;

  if (!file_chooser_type)
	{
		static const GTypeInfo file_chooser_info =
		{
			sizeof (GemFileChooserClass),
			NULL,		/* base_init */
			NULL,		/* base_finalize */
			(GClassInitFunc) gem_file_chooser_class_init,
			NULL,		/* class_finalize */
			NULL,		/* class_data */
			sizeof (GemFileChooser),
			0,			/* n_preallocs */
			(GInstanceInitFunc) gem_file_chooser_init,
		};

		file_chooser_type = g_type_register_static(GEM_TYPE_TREE_VIEW,
																						  "GemFileChooser",
																						  &file_chooser_info,
																						  0);
	}

	return file_chooser_type;
}

static void
gem_file_chooser_class_init (GemFileChooserClass * klass)
{
  GObjectClass	*object_class;

  object_class = G_OBJECT_CLASS(klass);
  parent_class = g_type_class_peek_parent(klass);

  object_class->dispose		= gem_file_chooser_dispose;
  object_class->finalize	= gem_file_chooser_finalize;

	klass->file_selected = gem_file_chooser_file_selected;
	klass->up_to_vroot   = gem_file_chooser_up_to_vroot;
	klass->enter_to_sub_dir = gem_file_chooser_enter_to_sub_dir;
 
	/* Action signals */
  file_chooser_signals[FILE_SELECTED] = 
	g_signal_new("file_selected",
							 G_OBJECT_CLASS_TYPE(klass),
							 G_SIGNAL_RUN_LAST,
							 G_STRUCT_OFFSET(GemFileChooserClass, file_selected),
							 NULL, NULL,
							 gtk_marshal_VOID__VOID,
							 G_TYPE_NONE,
							 0);
  file_chooser_signals[UP_TO_VROOT] = 
	g_signal_new("up_to_vroot",
							 G_OBJECT_CLASS_TYPE(klass),
							 G_SIGNAL_RUN_LAST,
							 G_STRUCT_OFFSET(GemFileChooserClass,up_to_vroot),
							 NULL, NULL,
							 gtk_marshal_VOID__VOID,
							 G_TYPE_NONE,
							 0);
  file_chooser_signals[ENTER_TO_SUB_DIR] = 
	g_signal_new("enter_to_sub_dir",
							 G_OBJECT_CLASS_TYPE(klass),
							 G_SIGNAL_RUN_LAST,
							 G_STRUCT_OFFSET(GemFileChooserClass, enter_to_sub_dir),
							 NULL, NULL,
							 gtk_marshal_VOID__VOID,
							 G_TYPE_NONE,
							 0);
	return;
}

static void
gem_file_chooser_dispose (GObject *object)
{
 	(*G_OBJECT_CLASS(parent_class)->dispose)(object);

 	return;
}

static void
gem_file_chooser_finalize (GObject *object)
{
	GemFileChooser * filechooser = GEM_FILE_CHOOSER(object);
	g_print("%s(): entering\n", __FUNCTION__);
	g_free(filechooser->current_path);
  (*G_OBJECT_CLASS(parent_class)->finalize)(object);
  g_hash_table_foreach(loaded_icons,free_a_hash_table_entry,NULL);
  g_hash_table_destroy ( loaded_icons );
  return;
}

static void
gem_file_chooser_file_info_print (GtkFileInfo		*info)
{
	struct tm loacl_time;
	time_t time;
	
	if (info == NULL)
	{
		return;
	}

	time = (time_t)gtk_file_info_get_modification_time(info);
	localtime_r(&time, &loacl_time);
	
	g_print("%s():Info 0x%08x\n",
					__FUNCTION__,
					(guint32)info);
	g_print("%s():|- Display Name  : %s\n",
					__FUNCTION__,
					gtk_file_info_get_display_name(info));
	g_print("%s():|- Display Key   : %s\n",
					__FUNCTION__,
					gtk_file_info_get_display_key(info));
	g_print("%s():|- Folder        : %s\n",
					__FUNCTION__,
					(gtk_file_info_get_is_folder(info) ? "TRUE" : "FALSE"));
	g_print("%s():|- Hidden        : %s\n",
					__FUNCTION__,
					(gtk_file_info_get_is_hidden(info) ? "TRUE" : "FALSE"));
	g_print("%s():|- MIME Type     : %s\n",
					__FUNCTION__,
					gtk_file_info_get_mime_type(info));
	g_print("%s():|- Modified Time : %04d/%02d/%02d %02d:%02d:%02d\n",
					__FUNCTION__,
					loacl_time.tm_year + 1900,
					loacl_time.tm_mon + 1,
					loacl_time.tm_mday,
					loacl_time.tm_hour,
					loacl_time.tm_min,
					loacl_time.tm_sec);
	g_print("%s():`- Size          : %lld\n",
					__FUNCTION__,
					gtk_file_info_get_size(info));
					
	g_print("\n");
	return;
}

static gint 
gemgem_file_chooser_sort_func (GtkTreeModel 		*model,
															 GtkTreeIter			*a,
															 GtkTreeIter			*b,
															 gpointer					 user_data)
{
	gint ret = 0;
	gchar * str1 = NULL;
	gchar * str2 = NULL;
	gboolean is_folder1 = FALSE;
	gboolean is_folder2 = FALSE;
	
	gtk_tree_model_get(model,
										 a,
										 GEM_FC_COL_NAME,
										 &str1,
										 GEM_FC_COL_IS_FOLDER,
										 &is_folder1,
										 -1);
	gtk_tree_model_get(model,
										 b,
										 GEM_FC_COL_NAME,
										 &str2,
										 GEM_FC_COL_IS_FOLDER,
										 &is_folder2,
										 -1);
	
	if (str1 == NULL && str2 == NULL)
	{
		ret = 0;
	}
	else if (str1 == NULL || str2 == NULL)
	{
		ret = (str1 == NULL) ? -1 : 1;
	}
	else
	{
    gboolean str1_is_sel_all=!strcmp(str1,"  Select All");
    gboolean str2_is_sel_all=!strcmp(str2,"  Select All");
    if(str1_is_sel_all&&str2_is_sel_all)
      ret = 0;
    if(str1_is_sel_all||str2_is_sel_all)
    {
      ret =  (str1_is_sel_all) ? -1 : 1;
    }
    if (is_folder1 != is_folder2)
		{
			ret = (is_folder1 > is_folder2) ? -1 : 1;
		}
		else
		{
			ret = g_utf8_collate(str1, str2);
		}
	}
	
	g_free(str1);
	g_free(str2);
	
	return ret;
}

static void
gem_file_chooser_file_selected (GemFileChooser	*filechooser)
{
	return;
}

static void
gem_file_chooser_up_to_vroot(GemFileChooser	*filechooser)
{
	return;
}
static void
gem_file_chooser_enter_to_sub_dir(GemFileChooser	*filechooser)
{
	return;
}

static void 
mark_toggled_cb(GtkCellRendererToggle *celltoggle,gchar *pathstr,gpointer userdata)
{
  GtkTreeIter iter;
  GtkTreeModel *model = NULL;
  gboolean active;
  GtkTreePath *path = NULL;
  GemFileChooser * filechooser = GEM_FILE_CHOOSER(userdata);
  
  if(gem_file_chooser_get_selectable==FALSE)
  {
   DBG("_____________________________________________Error BUG");
   return;
  }
  model = gem_tree_view_get_model(GEM_TREE_VIEW(GEM_FILE_CHOOSER(filechooser)));
	path = gtk_tree_path_new_from_string (pathstr);
  
  if (!gtk_tree_model_get_iter (model, &iter, path))
	{
	  g_print ("%s(): bad path %s\n", __FUNCTION__, pathstr);
	  return;
	}
  if (model)
  {
    
    /*first check if select all has been choosen*/     
    gtk_tree_model_get ( GTK_TREE_MODEL (model),
                         &iter,
                         GEM_FC_COL_CHECK_BOX,
                         &active,
                         -1);
    active = !active;
    gtk_list_store_set ( GTK_LIST_STORE (model),&iter,
                         GEM_FC_COL_CHECK_BOX, active, 
                         GEM_FC_COL_CHECK, active, 
                         -1);
    gtk_cell_renderer_toggle_set_active((GtkCellRendererToggle *)celltoggle,active);
    #if 1
    {
      GtkTreePath *root = NULL;
      root = gtk_tree_path_new_first ();
      if (!gtk_tree_path_compare (path, root))
      {
        if(active)
          gem_file_chooser_select_all(filechooser);
        else
          gem_file_chooser_unselect_all(filechooser);
      } 
      gtk_tree_path_free(root);
    }  
    #endif                    
  }
  return ;
}

#if 1
static 
gem_file_chooser_tree_view_row_activated (GtkTreeView				 	*treeview,
																					GtkTreePath					*path,
																					GtkTreeViewColumn		*col,
																					gpointer						 user_data)
{
	GemFileChooser * filechooser = GEM_FILE_CHOOSER(user_data);
	GtkTreeModel * model = NULL;
	GtkTreeIter iter;
  GnomeVFSFileInfo *vfs_file_info; /*= gnome_vfs_file_info_new ();*/
  gboolean selectable=gem_file_chooser_get_selectable(filechooser);
  if(selectable)
    return;
  model = gem_tree_view_get_model(GEM_TREE_VIEW(filechooser));
	
	if (model && gtk_tree_model_get_iter(model,
																			 &iter,
																			 path))
	{
		//GtkFilePath * filepath = NULL;
		GError * error = NULL;
		gchar * name = NULL;
		gboolean is_folder = FALSE;
		gtk_tree_model_get(model,
											 &iter,
											 GEM_FC_COL_NAME,
											 &name,
											 GEM_FC_COL_IS_FOLDER,
											 &is_folder,
											 GEM_FC_COL_VFS_FILE_INFO,
                       &vfs_file_info,
											 -1);
		g_print("%s(): %-20s [%s]\n",
						__FUNCTION__,
						name ? name : "null",
						(is_folder ? "FOLER" : "FILE"));

		if (name == NULL)
			return;
		if(name == "  My Collections")
      return;
    
    if (g_utf8_collate(name, "..") == 0)
		{
      gchar *temp = g_strdup(filechooser->current_path);
      if(strcmp(temp,virtual_root))
      {
        gchar *t = g_strrstr(temp,"/");
        if(t)
          *t='\0';
      }
      gchar *parent_str=g_strdup(temp);
      if(temp)
        g_free(temp);
			g_print("%s(): change to parent %s\n",
							__FUNCTION__,
							parent_str ? parent_str : "null");
			gem_file_chooser_set_current_path(filechooser,parent_str);
      
      gboolean isvroot = !strcmp (parent_str, virtual_root);
      if(isvroot)
	      g_signal_emit(G_OBJECT(filechooser),file_chooser_signals[UP_TO_VROOT],0);
      g_free(parent_str);
		}
		else/*not ..*/
		{
      /*change alias My Music to the real file name music*/
      /*for virtual root mechnism*/
      int i = 0;
      gboolean isvroot = !strcmp (filechooser->current_path, virtual_root);
      if (isvroot)
      {
        for (i = 0; i < VIRTUAL_TOPDIR_COUNT; i++)
        {
	        if (!strcmp (virtual_topdirs[i].alias, name))
          {  
            if(name)
            {
              g_free(name);
              name = g_strdup(virtual_topdirs[i].path);
            }
            break;
          }
        }
      }
      gchar * absolute_path = NULL;
			absolute_path = g_build_filename( filechooser->current_path,
																			  name,
																			  NULL );
    
      /*test if the file exists*/
      GnomeVFSURI *uri = gnome_vfs_uri_new(absolute_path);
      
      if (!gnome_vfs_uri_exists (uri))
      {
        g_print("non-exist uri");
        return;
      }
      gnome_vfs_uri_unref (uri);
			if (is_folder)
			{
        gem_file_chooser_set_current_path(filechooser,absolute_path);
        if(isvroot)
	        g_signal_emit(G_OBJECT(filechooser),file_chooser_signals[ENTER_TO_SUB_DIR],0);
			}
			else/*is_file*/
			{
	      g_signal_emit(G_OBJECT(filechooser),file_chooser_signals[FILE_SELECTED],0);
			}
			g_free(absolute_path);
		}
		g_free(name);
	}
	return;
}
#endif 
static void
gem_file_chooser_init (GemFileChooser * filechooser)
{
	GtkWidget * treeview = NULL;
	GtkListStore * liststore = NULL;
	GtkCellRenderer * renderer = NULL;
	GtkTreeViewColumn	* col = NULL;
  gem_resource_path = gem_env_get_resource_path();
	liststore = gtk_list_store_new(GEM_FC_NUMS_COLS,
																 G_TYPE_BOOLEAN,
																 GDK_TYPE_PIXBUF,
																 G_TYPE_STRING,
																 G_TYPE_BOOLEAN,
                                 G_TYPE_POINTER,
																 G_TYPE_BOOLEAN
                                 );
	treeview = gem_tree_view_get_view(GEM_TREE_VIEW(filechooser));
		
	
  /* column checkbox*/
  col = gtk_tree_view_column_new ();
  gtk_tree_view_column_set_title (col, "Check_Box");
  renderer = gtk_cell_renderer_toggle_new ();
  gtk_tree_view_column_pack_start (col, renderer, TRUE);
	//gtk_cell_renderer_set_fixed_size(renderer, 16, 16);
  gtk_tree_view_column_add_attribute (col, renderer, "active", GEM_FC_COL_CHECK_BOX);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), col);
  
  g_signal_connect (G_OBJECT (renderer), "toggled",G_CALLBACK (mark_toggled_cb),(gpointer)filechooser);
  gtk_cell_renderer_toggle_set_active((GtkCellRendererToggle*)renderer,FALSE);
  filechooser->check_box = col;  
  
  /* column pic */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, "Picutre");
	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), col);
		
	renderer = gtk_cell_renderer_pixbuf_new();
	gtk_cell_renderer_set_fixed_size(renderer,32 , 32);
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col,
																		 renderer,
																		 "pixbuf",
																		 GEM_FC_COL_PIC);
	
	/* column name */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, "Name");
	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), col);
		
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_renderer_set_fixed_size(renderer, 130, -1);
	g_object_set(G_OBJECT(renderer), "ypad", 5, NULL);
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col,
																		 renderer,
																		 "text",
																		 GEM_FC_COL_NAME);
	
	/* set model */	
	gem_tree_view_set_model(GEM_TREE_VIEW(filechooser),
												  GTK_TREE_MODEL(liststore));
	
	gem_tree_view_set_search_column(GEM_TREE_VIEW(filechooser),
																	GEM_FC_COL_NAME,
																	GTK_SORT_ASCENDING,
																	gemgem_file_chooser_sort_func);

	g_signal_connect(G_OBJECT(treeview),
									 "row-activated",
									 G_CALLBACK(gem_file_chooser_tree_view_row_activated),
									 (gpointer)filechooser);

  gem_file_chooser_set_selectable(filechooser,FALSE);
  gchar *env=getenv("GEM_FC_VROOT");
  if(env)
  {
    virtual_root = g_strdup(env);   
  }
  
  gem_file_chooser_set_current_path(filechooser,
																		virtual_root);
	return;
}

static void
gem_file_chooser_set_selection_all (GemFileChooser		*filechooser,
																		gboolean					 tag)
{
	GtkTreeModel * model = NULL;
	gint count = 0;
	gint i;

	if (filechooser->selectable == FALSE)
	{
		DBG("in non-selectable mode.\n");
		//return;
	}

	model = gem_tree_view_get_model(GEM_TREE_VIEW(filechooser));
	if (model == NULL)
	{
		return;
	}
	
	count = gtk_tree_model_iter_n_children(model, NULL);
	for (i = 1; i < count; i++)
	{
		GtkTreeIter iter;
				
		if (gtk_tree_model_iter_nth_child(model,
																			&iter,
																			NULL,
																			i) == TRUE)
		{
				gtk_list_store_set(GTK_LIST_STORE(model),
													 &iter,
													 GEM_FC_COL_CHECK,
													 tag,
													 GEM_FC_COL_CHECK_BOX,
													 tag,
													 -1);

		}
	}
	
	return;	
}

/* Public API
 */
GtkWidget*
gem_file_chooser_new (void)
{
  
  gnome_vfs_init();
  gnome_vfs_mime_info_reload ();
  return g_object_new(GEM_TYPE_FILE_CHOOSER,NULL);
  
}

static gboolean
do_scheduled_refresh (gpointer fchooser)
{
  DBG("Fresh");
  const gchar *current_path = gem_file_chooser_get_current_path (fchooser);
  gem_file_chooser_set_current_path (fchooser,current_path);
  refresh_scheduled = FALSE ;
  return FALSE;
}
void
directory_changed ( GnomeVFSMonitorHandle * handle,
   	                const gchar * monitor_uri,
  	                const gchar * info_uri,
  	                GnomeVFSMonitorEventType event_type, gpointer user_data )
{
  /* schedule timeout if necessary */
  if (!refresh_scheduled)
  {
    g_timeout_add (2000, (GSourceFunc) do_scheduled_refresh, user_data);
    refresh_scheduled = TRUE;
  }
}

/**
 * gem_file_chooser_set_current_path:
 * 
 * @filechooser	:	a #GemFileChooser object
 * @pathstr			:	a file path to set as the current path of
 * 								#GemFileChooser object
 **/
void
gem_file_chooser_set_current_path (GemFileChooser			*filechooser,
																	 const gchar				*pathstr)
{
  GdkPixbuf * icons[GEM_FC_NUMS_ICONS] = {
                                           NULL,
                                           NULL,
                                           NULL,
                                           NULL,
                                           NULL,
                                           NULL
                                          };
	
  GtkTreeModel * model = NULL;
  if (pathstr == NULL)
	{
		return;
	}
  
  gchar *current_path = g_strdup(pathstr);
  if(!strcmp(pathstr,"/"))
  {
    if(current_path)
    { 
      g_free((gchar *)current_path);
    }
    current_path = g_strdup(virtual_root);
  }
  if((g_str_has_prefix(pathstr,"vroot://"))&&(strlen(pathstr)>8))
  {
    gchar *temp_str=g_strdup(pathstr);
    temp_str=g_strdup(pathstr+8);
    current_path = g_strdup_printf("%s/%s",virtual_root,temp_str);
    g_free(temp_str);
  }
  
  GnomeVFSURI *uri = gnome_vfs_uri_new(current_path);
  if (!gnome_vfs_uri_exists (uri))
  {
    g_print("non-exist uri\n");
    return;
  }
  gnome_vfs_uri_unref (uri);
  
  model = gem_tree_view_get_model(GEM_TREE_VIEW(filechooser));
	if (model)
	{
		GSList * iter = NULL;
    loaded_icons = g_hash_table_new (g_str_hash, g_str_equal);
    
    /*free the pre tree model*/
    {
#if 1
      GtkTreeIter iter;
      gchar *name=NULL;
      gint i=1;
      GdkPixbuf *pixbuf=NULL;
      GnomeVFSFileInfo *vfs_file_info = gnome_vfs_file_info_new();
			gint count = gtk_tree_model_iter_n_children(model, NULL);
			for (i = 1; i < count; i++)
			{
				GtkTreeIter iter;
				if (gtk_tree_model_iter_nth_child(model,
																					&iter,
																					NULL,
																					i) == TRUE)
        {
          gtk_tree_model_get( GTK_TREE_MODEL(model),
					  								  &iter,
						  						    GEM_FC_COL_NAME,
                              &name, 
								  						GEM_FC_COL_PIC,
									  					&pixbuf,
                              GEM_FC_COL_VFS_FILE_INFO,
                              &vfs_file_info,
												  	  -1);
					if (name)
          {
            g_free(name);
          }
	        gdk_pixbuf_unref (pixbuf);
          gnome_vfs_file_info_unref(vfs_file_info);    
        }
      }
#endif
      gtk_list_store_clear(GTK_LIST_STORE(model));
    }
    
    /*add monitor to the current directory*/
    if (TRUE)
		{
    /* monitor current directory */
      if (dir_monitor)
        gnome_vfs_monitor_cancel (dir_monitor);
      if (gnome_vfs_monitor_add ( &dir_monitor, current_path,
			                            GNOME_VFS_MONITOR_DIRECTORY,
                                  (GnomeVFSMonitorCallback) directory_changed,
			                            filechooser ) != GNOME_VFS_OK )
      { 
        dir_monitor = NULL;
      }
      else
      {
        DBG("curre_path monitor  added to%s....",current_path);
      }
    } 
    if (TRUE)
		{
			gchar * fname[] = {
		                   		"music.png",
                   				"photo.png",
                  				"ringtone.png",
                  				"video.png",
                  				"sd.png",
                  				"parent.png",
                          "collection.png",
                          "normal.png"
                  			};
			gint i;
			for (i = 0; i < GEM_FC_NUMS_ICONS; i++)
			{
				gchar * picpath = NULL;
				picpath = g_build_filename( gem_resource_path,
																	  "pixmaps",
																	  fname[i],
																	  NULL 
                                  );
				if (gem_resource_load_pic( picpath, GEM_PIC_NORMAL) == TRUE )
				{
					icons[i] = gem_resource_find_pic( picpath, GEM_PIC_NORMAL );
				}
				g_free(picpath);
			}
		}
		/*
		 * we need a ".." item to act as
		 * the parent of current path, to supply 
		 * a possiblity to user to go backward
		 */
    
    if (strcmp (current_path, virtual_root))
		{
      GtkTreeIter iter;
	   	gtk_list_store_append(GTK_LIST_STORE(model),&iter);
		  gtk_list_store_set( GTK_LIST_STORE(model),
			        					  &iter,
							        	  GEM_FC_COL_NAME,
					         			  "..",
								          GEM_FC_COL_PIC,
		       							  icons[PARENT],
					        			  GEM_FC_COL_IS_FOLDER,
								          TRUE,
						        		  GEM_FC_COL_CHECK_BOX,
                          FALSE,
								          -1
                         );
		}
    else
    {
      GtkTreeIter iter;
	   	gtk_list_store_append(GTK_LIST_STORE(model),&iter);
		  gtk_list_store_set( GTK_LIST_STORE(model),
			        					  &iter,
							        	  GEM_FC_COL_NAME,
					         			  "  My Collections ",
								          GEM_FC_COL_PIC,
		       							  icons[COLLECTION],
					        			  GEM_FC_COL_IS_FOLDER,
								          TRUE,
						        		  GEM_FC_COL_CHECK_BOX,
								          FALSE,
                          -1
                         );
    }
    {  
          
      GnomeVFSDirectoryHandle *handle = NULL;
      GnomeVFSFileInfo *vfs_file_info = NULL;
	    GtkTreeIter  tree_iter;
      gchar *name;
      GnomeVFSResult result = GNOME_VFS_OK, open_dir_result;
      open_dir_result =gnome_vfs_directory_open ( &handle, current_path,
			                                             GNOME_VFS_FILE_INFO_DEFAULT
                                   		 	           | GNOME_VFS_FILE_INFO_FOLLOW_LINKS
                                                   | GNOME_VFS_FILE_INFO_GET_MIME_TYPE );

      while (open_dir_result == GNOME_VFS_OK)
      {
        vfs_file_info = gnome_vfs_file_info_new ();
        result = gnome_vfs_directory_read_next (handle, vfs_file_info);
        if (result != GNOME_VFS_OK)
	      {
          gnome_vfs_file_info_unref(vfs_file_info);
          gnome_vfs_directory_close (handle);
	        break;
	      }
        if (vfs_file_info->name != NULL && vfs_file_info->name[0] != '.')
	      {
          GdkPixbuf *pixbuf = NULL;
          GdkPixbuf *pb;
          gchar *mime_icon;
          gchar *absolute_path=g_strdup_printf("%s/%s",current_path,vfs_file_info->name);
          /*add virtual root mechnism*/ 
          int i = 0;
          if (!strcmp (current_path, virtual_root))
          {
            gtk_tree_view_column_set_visible(filechooser->check_box ,FALSE); 
            for (i = 0; i < VIRTUAL_TOPDIR_COUNT; i++)
            {
	            if (!strcmp (virtual_topdirs[i].path, vfs_file_info->name))
              {  
                name = g_strdup(virtual_topdirs[i].alias);
                break;
              }
            }
            if(i == VIRTUAL_TOPDIR_COUNT)
            {
              gnome_vfs_file_info_unref(vfs_file_info);
              continue;  
            }
            switch(i)
            {
               case  MUSIC:
                     pixbuf = gdk_pixbuf_ref(icons[MUSIC]);
                     pb = gdk_pixbuf_scale_simple (pixbuf, 24, 24, GDK_INTERP_BILINEAR);
                     break;
                   
               case  PHOTO:
                     pixbuf = gdk_pixbuf_ref(icons[PHOTO]);
                     pb = gdk_pixbuf_scale_simple (pixbuf, 24, 24, GDK_INTERP_BILINEAR);
                     break;
               case  VIDEO:
                     pixbuf = gdk_pixbuf_ref(icons[VIDEO]);
                     pb = gdk_pixbuf_scale_simple (pixbuf, 24, 24, GDK_INTERP_BILINEAR);
                     break;
               case  RINGTONE :
                     pixbuf = gdk_pixbuf_ref(icons[RINGTONE]);
                     pb = gdk_pixbuf_scale_simple (pixbuf, 24, 24, GDK_INTERP_BILINEAR);
                     break;
               case  SD :
                     pixbuf = gdk_pixbuf_ref(icons[SD]);
                     pb = gdk_pixbuf_scale_simple (pixbuf, 24, 24, GDK_INTERP_BILINEAR);
                     break;
               case  COLLECTION :
                     pixbuf = gdk_pixbuf_ref(icons[COLLECTION]);
                     pb = gdk_pixbuf_scale_simple (pixbuf,32, 32, GDK_INTERP_BILINEAR);
                     break;
               default :
                     pixbuf = gdk_pixbuf_ref(icons[NORMAL]);
                     break;
            }
          }//not virtul root
          else 
          /*add  icons  find icons in mime database*/
          {
            name = g_strdup(vfs_file_info->name);
            gchar *mime_icon =  find_mime_icon(vfs_file_info,absolute_path);
            pixbuf =  get_pixbuf(mime_icon);
            pb = gdk_pixbuf_scale_simple (pixbuf, 24, 24, GDK_INTERP_BILINEAR);
          }
          
          g_free(absolute_path);
         
          gtk_list_store_append(GTK_LIST_STORE(model), &tree_iter);

          if (vfs_file_info->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
          {
            gtk_list_store_set(GTK_LIST_STORE(model),
										       		 &tree_iter,
											   	  	 GEM_FC_COL_NAME,
														   name,
															 GEM_FC_COL_PIC,
															 pb,
															 GEM_FC_COL_IS_FOLDER,
															 TRUE,
															 GEM_FC_COL_VFS_FILE_INFO,
                               vfs_file_info,
															 GEM_FC_COL_CHECK_BOX,
                               FALSE,
															 -1);
					}
					else
					{
					  gtk_list_store_set(GTK_LIST_STORE(model),
															 &tree_iter,
															 GEM_FC_COL_NAME,
															 name,
															 GEM_FC_COL_PIC,
															 pb,
															 GEM_FC_COL_IS_FOLDER,
															 FALSE,
															 GEM_FC_COL_VFS_FILE_INFO,
                               vfs_file_info,
															 GEM_FC_COL_CHECK_BOX,
                               FALSE,
															 -1);
          }           
        }
        else
        {
          gnome_vfs_file_info_unref(vfs_file_info);
          continue;    
        }
      }//end of while 
      /*if open fail*/
      if (open_dir_result != GNOME_VFS_OK)
      {
        gchar * error=NULL;
        switch (open_dir_result)
	      {
          case GNOME_VFS_ERROR_READ_ONLY:
	             error = g_strdup (_("Destination is read only."));
     	         break;
	        case GNOME_VFS_ERROR_ACCESS_DENIED:
	             error = g_strdup (_("Access denied."));
	             break;
	        case GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM:
	             error = g_strdup (_("Read only file system."));
	             break;
	        case GNOME_VFS_ERROR_NOT_FOUND:
               break;
	        case GNOME_VFS_ERROR_GENERIC:
	             break;
	        default:
               error = g_strdup_printf ("%s %s", _("Error:"), gnome_vfs_result_to_string(open_dir_result));
	        break;
        }
        if(error)
        {
          g_print("%s",error);
          g_free (error);
        }
      }
	  }
	}
	/*
	 * log the current path and update
	 * tree view cursor to the top line
	 */
	if (filechooser->current_path != NULL)
	{
		g_free(filechooser->current_path);
	}
	filechooser->current_path = g_strdup(current_path);
	if (TRUE)
	{
		GtkWidget * view = NULL;
		
		view = gem_tree_view_get_view(GEM_TREE_VIEW(filechooser));
		if (view)
		{
			GtkTreePath * tree_path = NULL;
			GtkTreeViewColumn * col = NULL;
			
			tree_path = gtk_tree_path_new_first();
			col = gtk_tree_view_get_column(GTK_TREE_VIEW(view),
																		 0);
															 
			gtk_tree_view_set_cursor(GTK_TREE_VIEW(view),
															 tree_path,
															 col,
															 FALSE);
			gtk_tree_path_free(tree_path);
		}
		
	}
	return;
}

G_CONST_RETURN gchar *
gem_file_chooser_get_current_path (GemFileChooser			*filechooser)
{
  return filechooser->current_path;
}

void
gem_file_chooser_set_selectable (GemFileChooser			*filechooser,
																 gboolean						 selectable)
{
	gboolean sel = gem_file_chooser_get_selectable(filechooser);
  if(sel == selectable)
  {
    return;
  }
  GtkWidget * treeview = NULL;
  const gchar *current_path = NULL;
  current_path = gem_file_chooser_get_current_path(filechooser);
  if(current_path == NULL)
    return;
  if (!strcmp (current_path, virtual_root))
  {
    g_print("%s():System Files Can Be Marked\n",__FUNCTION__); 
    gtk_tree_view_column_set_visible(filechooser->check_box ,FALSE); 
    return;
  }
  else
    gtk_tree_view_column_set_visible(filechooser->check_box ,selectable); 
  /*cusor focus set*/   
	if (TRUE)
	{
		GtkWidget * view = NULL;
		
		view = gem_tree_view_get_view(GEM_TREE_VIEW(filechooser));
		if (view)
		{
			GtkTreePath * tree_path = NULL;
			GtkTreeViewColumn * col = NULL;
			
			tree_path = gtk_tree_path_new_first();
			if(selectable)
        col = gtk_tree_view_get_column(GTK_TREE_VIEW(view),
																		 0);
			else
        col = gtk_tree_view_get_column(GTK_TREE_VIEW(view),
																		 1);
			gtk_tree_view_set_cursor( GTK_TREE_VIEW(view),
															  tree_path,
															  col,
															  FALSE
                              );
			gtk_tree_path_free(tree_path);
		}
	}
#if 1
  /*set ".." as "select all",and change it's icon*/
	GtkTreeModel * model = NULL;
  model = gem_tree_view_get_model(GEM_TREE_VIEW(filechooser));
  GtkTreeIter *tree_iter = NULL;
  gboolean result = gtk_tree_model_get_iter_first(model,&tree_iter);
 
  GdkPixbuf * pixbuf = NULL;
	gchar * picpath = NULL;
  
  if(selectable)
  {
		picpath = g_build_filename(gem_resource_path,
																		 "pixmaps",
																		 "file_selected.png",
																		 NULL);
		if (gem_resource_load_pic(picpath,GEM_PIC_NORMAL) == TRUE)
		{
			pixbuf = gem_resource_find_pic(picpath,GEM_PIC_NORMAL);
		}
	  gtk_list_store_set( GTK_LIST_STORE(model),
												 &tree_iter,
												 GEM_FC_COL_PIC,
												 pixbuf,
												 GEM_FC_COL_NAME,
												 "  Select All",
                         GEM_FC_COL_CHECK,
                         FALSE,
                         GEM_FC_COL_CHECK_BOX,
                         FALSE,
                         -1
                       );
    
  }
  else
  {
		picpath = g_build_filename(gem_resource_path,
																		 "pixmaps",
																		 "parent.png",
																		 NULL);
		if (gem_resource_load_pic(picpath,GEM_PIC_NORMAL) == TRUE)
		{
			pixbuf = gem_resource_find_pic(picpath,GEM_PIC_NORMAL);
		}
	  gtk_list_store_set(GTK_LIST_STORE(model),
														 &tree_iter,
														 GEM_FC_COL_PIC,
														 pixbuf,
														 GEM_FC_COL_NAME,
														 "..",
                             -1);
    gem_file_chooser_unselect_all(filechooser);
    
  }        
    g_free(picpath);
#endif 
   if (filechooser->selectable == selectable)
	 {
     return;
	 }
   filechooser->selectable = selectable;
	 return;
}

gboolean
gem_file_chooser_get_selectable (GemFileChooser			*filechooser)
{
	return filechooser->selectable;
}

GList *
gem_file_chooser_get_current_selections (GemFileChooser		*filechooser)
{
	GtkTreeModel * model = NULL;
	GList * results = NULL;
	gint count = 0;
	gint i;

	model = gem_tree_view_get_model(GEM_TREE_VIEW(filechooser));
	if (model == NULL)
	{
		return results;
	}
	
	if (filechooser->selectable == FALSE)
	{
		GtkWidget * treeview = NULL;

		treeview = gem_tree_view_get_view(GEM_TREE_VIEW(filechooser));
		if (treeview)
		{
			GtkTreePath * path = NULL;
			gtk_tree_view_get_cursor(GTK_TREE_VIEW(treeview),
															 &path,
															 NULL);
			if (path)
			{
				GtkTreeIter iter;
			  /*if the first then ,return NULL*/
        GtkTreePath *root = gtk_tree_path_new_first ();
        if (!gtk_tree_path_compare (path, root))
        {
          return results;
        }
				if (gtk_tree_model_get_iter(model,
																		&iter,
																		path) == TRUE)
				{
					gchar * fname = NULL;
          gtk_tree_model_get(GTK_TREE_MODEL(model),
														 &iter,
														 GEM_FC_COL_NAME,
														 &fname,
														 -1);
				
					results = g_list_append(results,
																	(gpointer)fname);
					
				}
				gtk_tree_path_free(path);
			}
		}
		
		return results;
	}

	count = gtk_tree_model_iter_n_children(model, NULL);
	for (i = 1; i < count; i++)
	{
		GtkTreeIter iter;
				
		if (gtk_tree_model_iter_nth_child(model,
																			&iter,
																			NULL,
																			i) == TRUE)
		{
			gchar * fname = NULL;
			gboolean selected = FALSE;
			
			gtk_tree_model_get(GTK_TREE_MODEL(model),
												 &iter,
												 GEM_FC_COL_NAME,
												 &fname,
												 GEM_FC_COL_CHECK,
												 &selected,
												 -1);
												 
			if(selected == TRUE)
			{
				results = g_list_append(results,
																(gpointer)fname);
			}
			
		}
	}
	
	return results;	
}


GList *
gem_file_chooser_get_current_selections_vfs_file_info (GemFileChooser		*filechooser)
{
	GtkTreeModel * model = NULL;
	GList * results = NULL;
	gint count = 0;
	gint i;

	model = gem_tree_view_get_model(GEM_TREE_VIEW(filechooser));
	if (model == NULL)
	{
		return results;
	}
	
	if (filechooser->selectable == FALSE)
	{
		GtkWidget * treeview = NULL;

		treeview = gem_tree_view_get_view(GEM_TREE_VIEW(filechooser));
		if (treeview)
		{
			GtkTreePath * path = NULL;
			gtk_tree_view_get_cursor(GTK_TREE_VIEW(treeview),
															 &path,
															 NULL);
			if (path)
			{
				GtkTreeIter iter;
				
				if (gtk_tree_model_get_iter(model,
																		&iter,
																		path) == TRUE)
				{
				
          GnomeVFSFileInfo *vfs_file_info; /*= gnome_vfs_file_info_new ();*/
					gtk_tree_model_get(GTK_TREE_MODEL(model),
														 &iter,
														 GEM_FC_COL_VFS_FILE_INFO,
														 &vfs_file_info,
														 -1);
				
					results = g_list_append(results,
																	(gpointer)vfs_file_info);
					
				}
				gtk_tree_path_free(path);
			}
		}
		
		return results;
	}

	count = gtk_tree_model_iter_n_children(model, NULL);
	for (i = 1; i < count; i++)
	{
		GtkTreeIter iter;
				
		if (gtk_tree_model_iter_nth_child(model,
																			&iter,
																			NULL,
																			i) == TRUE)
		{
			
      gboolean selected = FALSE;
			
      GnomeVFSFileInfo *vfs_file_info; /*= gnome_vfs_file_info_new ();*/
			gtk_tree_model_get(GTK_TREE_MODEL(model),
												 &iter,
												 GEM_FC_COL_VFS_FILE_INFO,
												 &vfs_file_info,
												 GEM_FC_COL_CHECK,
												 &selected,
												 -1);
												 
			if(selected == TRUE)
			{
				results = g_list_append(results,
																(gpointer)vfs_file_info);
			}
			
		}
	}
	
	return results;	
}
void
gem_file_chooser_select_all (GemFileChooser		*filechooser)
{
	gem_file_chooser_set_selection_all(filechooser, TRUE);
  return;
}

void
gem_file_chooser_unselect_all (GemFileChooser		*filechooser)
{
	gem_file_chooser_set_selection_all(filechooser, FALSE);
  return;
}

/*
vi:ts=2:nowrap:ai:expandtab 
*/
