/*	libgemwidget - Gtk+ Embedded Widget library
 * 
 *	gemtranswindow.c - Translucnet Window object routines
 *
 *	Authors: YE Nan <nan.ye@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */
#include <stdio.h>
#include <stdlib.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/extensions/Xcomposite.h>
#include <X11/extensions/Xdamage.h>
#include <X11/extensions/Xrender.h>

#include <glib-object.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gdk/gdkkeysyms.h>

#include "gemxwindowentity.h"
#include "gemtranswindow.h"

#define 	DETAIL_DEBUG	0

typedef struct _GemTranslucentWindowPriv	GemTranslucentWindowPriv;

struct _GemTranslucentWindowPriv
{
	gint				 xdamage_event_base;
	gdouble			 opacity;
	gboolean		 fake_trans;
	
	GList				*window_entities;
};

#define GEM_TRANSLUCENT_WINDOW_GET_PRIVATE(obj)	(G_TYPE_INSTANCE_GET_PRIVATE((obj), \
																								 GEM_TYPE_TRANSLUCENT_WINDOW, \
																								 GemTranslucentWindowPriv));

enum {
	PROP_GEM_TRANSLUCENT_WINDOW_0,
	PROP_OPACITY,
	PROP_FAKE_TRANS,
};

enum {
	GEM_TRANSLUCENT_WINDOW_LAST_SIGNAL,
};

static void		gem_translucent_window_class_init				(GemTranslucentWindowClass			*klass);
static void		gem_translucent_window_init							(GemTranslucentWindow						*window);
static void		gem_translucent_window_finalize					(GObject												*object);

static void		gem_translucent_window_set_property			(GObject												*object,
																											 guint													 prop_id,
																											 const GValue										*value,
																											 GParamSpec											*pspec);
static void		gem_translucent_window_get_property			(GObject												*object,
																											 guint													 prop_id,
																											 GValue													*value,
																											 GParamSpec											*pspec);

static GdkFilterReturn gem_translucent_window_xevent_filter 	(GdkXEvent        		  *xevent,
																															 GdkEvent								*event,
																															 GemTranslucentWindow		*window);

static GtkWindowClass *parent_class = NULL;
//static guint trans_window_signals[GEM_TRANSLUCENT_WINDOW_LAST_SIGNAL];

GType
gem_translucent_window_get_type (void)
{
	static GType view_type = 0;

  if (!view_type)
	{
		static const GTypeInfo view_info =
		{
			sizeof (GemTranslucentWindowClass),
			NULL,		/* base_init */
			NULL,		/* base_finalize */
			(GClassInitFunc) gem_translucent_window_class_init,
			NULL,		/* class_finalize */
			NULL,		/* class_data */
			sizeof (GemTranslucentWindow),
			0,			/* n_preallocs */
			(GInstanceInitFunc) gem_translucent_window_init,
			NULL,
		};
		
		view_type = g_type_register_static(GTK_TYPE_WINDOW,
																			 "GemTranslucentWindow",
																			 &view_info,
																			 0);
	}
	
	return view_type;
}

static Picture
_gem_translucent_window_get_picture_from_drawable (GdkDrawable *drawable)
{
	Picture picture;
	GdkVisual *visual;
	XRenderPictFormat *format;
	XRenderPictureAttributes pa = {0};

  g_return_val_if_fail(GDK_IS_DRAWABLE(drawable), None);

  visual = gdk_drawable_get_visual(drawable);

  format = XRenderFindVisualFormat(GDK_DISPLAY (),
																	 GDK_VISUAL_XVISUAL (visual));
	g_return_val_if_fail(format, None);

  pa.subwindow_mode = IncludeInferiors;
  pa.repeat = False;

  picture = 
  	XRenderCreatePicture(GDK_DISPLAY(),
												 GDK_DRAWABLE_XID(drawable),
												 format,
												 CPSubwindowMode | CPRepeat,
												 &pa);

  return picture;
}

static Picture
_gem_translucent_window_get_picture_from_color (const GdkColor *gcolor,
																								guint           width,
																								guint						height,
																								guint						depth,
																								gdouble					alpha)
{
	Display *display = GDK_DISPLAY();
	gint screen = DefaultScreen(display);
  XRenderPictFormat *format = NULL;
  XRenderPictureAttributes pa = {0};
  XRenderColor color = {0};
  Picture picture;
  Pixmap pixmap;
  gint pict_format = PictStandardA8;
  gboolean def_pict_format = FALSE;

  g_return_val_if_fail(gcolor, None);

  pixmap = 
  	XCreatePixmap(display,
									DefaultRootWindow(display),
									width,
									height,
									depth);

  pa.repeat = True;
  
  switch (depth)
  {
  	case 32:
  		pict_format = PictStandardARGB32;
  		break;
  		
  	case 24:
  		pict_format = PictStandardRGB24;
  		break;
  	
  	case 8:
  		pict_format = PictStandardA8;
  		break;
  		
  	default: 
//  		pict_format = PictStandardA8;
  		def_pict_format = TRUE;
  		break;
  } 
  
  if (def_pict_format)
  {
 		format = 
				XRenderFindVisualFormat(display,
																DefaultVisual(display, screen));
  }
  else
  { 
		format = XRenderFindStandardFormat(display, pict_format);
  }
  g_return_val_if_fail(format, None);
  
  picture = 
  	XRenderCreatePicture(display,
												 pixmap,
												 format,
												 CPRepeat,
												 &pa);

  color.alpha = alpha * 0xFFFF;
  color.red   = gcolor->red;
  color.green = gcolor->green;
  color.blue  = gcolor->blue;

	XRenderFillRectangle(GDK_DISPLAY (),
											 PictOpSrc,
											 picture,
											 &color,
											 0, 0,
											 width, height);

  XFreePixmap(display, pixmap);

  return picture;
}

static Picture
_gem_translucent_window_render_background_picture (GemTranslucentWindow *window)
{
	GemTranslucentWindowPriv *priv = GEM_TRANSLUCENT_WINDOW_GET_PRIVATE(window);
	Display *display = GDK_DISPLAY();
	gint screen;
	Picture	bg_picture = None;
	const GdkColor white = {
  	0xFFFF, 
    0xFFFF,
    0xFFFF,
    0xFFFF
	};
	gint tw_x, tw_y;
	gint tw_w, tw_h;
	
	tw_w = GTK_WIDGET(window)->allocation.width;
	tw_h = GTK_WIDGET(window)->allocation.height;
	gdk_window_get_root_origin(GTK_WIDGET(window)->window,
														 &tw_x,
														 &tw_y);
	
	screen = DefaultScreen(display);

	bg_picture =
		_gem_translucent_window_get_picture_from_color(&white, 
																									 tw_w,
																									 tw_h,
																									 DefaultDepth(display, screen),
																									 1.0);
/*	g_print("%s(): bg_picture(%p), depth(%d)\n",
					__FUNCTION__,
					bg_picture,
					DefaultDepth(display, screen));
*/					
 	if (priv->window_entities)
 	{
 		GList *iter;
  	for (iter = priv->window_entities; iter ; iter = g_list_next(iter))
  	{
  		GemXWindowEntity *entity = GEM_XWINDOW_ENTITY(iter->data);
			XWindowAttributes *attr;
			Picture picture = None;
			Picture alpha_picture = None;
			Window win;
			guint render_type = GEM_XWIN_RENDER_NOP;
 			gint src_x, src_y;
 			gint dest_x, dest_y;
 			gint dest_width, dest_height;
			const GdkColor alpha_color = {
				0xFFFF,
				0x0,
				0x0,
				0x0
			};
				  
			g_object_get(G_OBJECT(entity),
									 "handler",
									 &win,
									 "attribute",
									 &attr,
									 "render-type",
									 &render_type,
									 "picture",
									 &picture,
									 NULL);
			
			if ((attr->x + attr->width) <= tw_x ||
					(attr->y + attr->height) <= tw_y || 
					(tw_x + tw_w) <= attr->x ||
					(tw_y + tw_h) <= attr->y)
			{
				continue;				
			}
			
			if (render_type == GEM_XWIN_RENDER_NOP)
			{
				continue;
			}
			
			if (render_type == GEM_XWIN_RENDER_TRANS)
			{
				alpha_picture =
					_gem_translucent_window_get_picture_from_color(&alpha_color, 
																												 tw_w,
																												 tw_h,
																												 8,
																												 0.5);
			}

#if DETAIL_DEBUG			
			g_print("%s(): src (%3d, %3d) - %3d x %3d\n",
							__FUNCTION__,
							attr->x,
							attr->y,
							attr->width,
							attr->height);

			g_print("%s(): dest to (%3d, %3d) - %3d x %3d\n",
							__FUNCTION__,
							tw_x,
							tw_y,
							tw_w,
							tw_h);
#endif

			src_x = MAX(attr->x, tw_x) - attr->x;
			src_y = MAX(attr->y, tw_y) - attr->y;
			
			dest_x = MAX(attr->x, tw_x) - tw_x;
			dest_y = MAX(attr->y, tw_y) - tw_y;
			
			dest_width = 
				MIN((attr->x + attr->width), (tw_x + tw_w)) - 
				MAX(attr->x, tw_x); 
			dest_height =
				MIN((attr->y + attr->height), (tw_y + tw_h)) - 
				MAX(attr->y, tw_y); 

#if DETAIL_DEBUG				
			g_print("%s(): composite from (%3d, %3d) to (%3d, %3d) - %3d x %3d\n\n",
							__FUNCTION__,
							src_x,
							src_y,
							dest_x,
							dest_y,
							dest_width,
							dest_height);
#endif
		
			XRenderComposite(display, 
											 (alpha_picture == None) ? PictOpSrc : PictOpOver,
											 picture,
											 alpha_picture,
											 bg_picture,
											 src_x, src_y,
											 src_x, src_y, 
											 dest_x, dest_y,
											 dest_width, dest_height);
  			
			if (alpha_picture != None)
			{						 
				XRenderFreePicture(display, alpha_picture);
			}
  	}
 	}
 	
	return bg_picture;
}

static gboolean 
gem_translucent_window_expose_event (GtkWidget				*widget,
																		 GdkEventExpose		*event)
{
	GemTranslucentWindowPriv *priv = GEM_TRANSLUCENT_WINDOW_GET_PRIVATE(widget);
	gboolean result;

#if DETAIL_DEBUG	
  g_print("%s(): %s(window: %p) exposed, area [%3d, %3d], %3d x %3d\n",
  				__FUNCTION__,
					gtk_widget_get_name(widget),
					widget->window,
					event->area.x,
					event->area.y,
					event->area.width,
					event->area.height);
#endif

  if (GTK_WIDGET_DRAWABLE (widget) && 
  		priv->opacity != 1.0)
  {
		GdkDrawable *drawable;
		gint x_offset, y_offset;
		Picture picture;
		Picture render_picture;
		Picture alpha_picture = None;

		gdk_window_get_internal_paint_info(widget->window,
																			 &drawable,
																			 &x_offset,
																			 &y_offset);

		picture = _gem_translucent_window_get_picture_from_drawable(drawable);
		g_return_val_if_fail(picture, FALSE);

		g_object_set_data(G_OBJECT(drawable),
											"picture",
											GINT_TO_POINTER(picture));
		
		render_picture = 
			_gem_translucent_window_render_background_picture(GEM_TRANSLUCENT_WINDOW(widget));
		
		if (priv->opacity < 1.0 &&
				priv->opacity >= 0.0)
		{
			const GdkColor alpha_color = {
				0xFFFF,
				0x0,
				0x0,
				0x0
			};
			
			alpha_picture = 
				_gem_translucent_window_get_picture_from_color(&alpha_color, 
																											 event->area.width,
																											 event->area.height,
																											 8,
																											 (1.0 - priv->opacity));
		}
		
    if (render_picture != None)
    {
    	gint base_x, base_y;
    	
    	gdk_window_get_root_origin(widget->window,
    														 &base_x,
    														 &base_y);
			XRenderComposite(GDK_DISPLAY(),
											 (alpha_picture == None) ? PictOpSrc : PictOpOver,
											 render_picture,
											 alpha_picture,
											 picture,
											 event->area.x, 
											 event->area.y,
//											 event->area.x, 
//											 event->area.y,
											 0,
											 0,
											 event->area.x - x_offset,
											 event->area.y - y_offset,
											 event->area.width,
											 event->area.height);
    }
    

		result = 
			GTK_WIDGET_CLASS(parent_class)->expose_event(widget,
																									 event);

		if (alpha_picture)
		{
			XRenderFreePicture(GDK_DISPLAY(),
												 alpha_picture);
		}
		
		XRenderFreePicture(GDK_DISPLAY(),
											 render_picture);

		XRenderFreePicture(GDK_DISPLAY(),
											 picture);

		g_object_set_data(G_OBJECT(drawable),
											"picture",
											GINT_TO_POINTER(None));

		return result;
	}

	return GTK_WIDGET_CLASS(parent_class)->expose_event(widget,
																											event);;
}

static gboolean
gem_translucent_window_map_event (GtkWidget		*widget,
																	GdkEventAny	*event)
{
	GemTranslucentWindowPriv *priv = GEM_TRANSLUCENT_WINDOW_GET_PRIVATE(widget);
	
	GTK_WIDGET_CLASS(parent_class)->map_event(widget, event);
				
	XCompositeRedirectSubwindows(GDK_DISPLAY(),
															 GDK_DRAWABLE_XID(widget->window),
															 CompositeRedirectAutomatic);
	
	return FALSE;
}

static gboolean
gem_translucent_window_unmap_event (GtkWidget		*widget,
																		GdkEventAny	*event)
{
	GemTranslucentWindowPriv *priv = GEM_TRANSLUCENT_WINDOW_GET_PRIVATE(widget);

	XCompositeRedirectSubwindows(GDK_DISPLAY(),
															 GDK_DRAWABLE_XID(widget->window),
															 CompositeRedirectAutomatic);
														 
	return GTK_WIDGET_CLASS(parent_class)->unmap_event(widget, event);
}

static void
gem_translucent_window_class_init (GemTranslucentWindowClass * klass)
{
	GtkWidgetClass *widget_class;
  GObjectClass *object_class;
  GParamSpec *opacity_param;
	GParamSpec *fake_trans_param;
 
	parent_class = g_type_class_peek_parent(klass);
  object_class = G_OBJECT_CLASS(klass);
  widget_class = GTK_WIDGET_CLASS(klass);
	
	opacity_param =
		g_param_spec_double("opacity", 
												"Opacity",
												"Window Opacity",
												0.0,
												1.0,
												1.0,
												G_PARAM_READWRITE);

	fake_trans_param =
		g_param_spec_boolean("fake-translucent",
												 "Fake Translucent",
												 "Fake Translucent",
												 TRUE,
												 G_PARAM_READWRITE);
												
  object_class->finalize			= gem_translucent_window_finalize;
	object_class->set_property	= gem_translucent_window_set_property;
	object_class->get_property	= gem_translucent_window_get_property;

	widget_class->expose_event	= gem_translucent_window_expose_event;
	widget_class->map_event			= gem_translucent_window_map_event;
	widget_class->unmap_event		= gem_translucent_window_unmap_event;
	
	g_type_class_add_private(klass,
													 sizeof(GemTranslucentWindowPriv));

	g_object_class_install_property(object_class, 
																	PROP_OPACITY,
																	opacity_param);
																	
	g_object_class_install_property(object_class, 
																	PROP_FAKE_TRANS,
																	fake_trans_param);
																	
	return;
}

static void
gem_translucent_window_finalize (GObject *object)
{
	GemTranslucentWindowPriv *priv = GEM_TRANSLUCENT_WINDOW_GET_PRIVATE(object);

	if (priv->window_entities)
	{
		GList *iter;
		
		for (iter = priv->window_entities; iter; iter = g_list_next(iter))
		{
			g_object_unref(iter->data);
		}
		
		g_list_free(priv->window_entities);
	}
	
	G_OBJECT_CLASS(parent_class)->finalize(object);

	return;
}

static void
gem_translucent_window_set_property (GObject				*object,
																		 guint					 prop_id,
																		 const GValue		*value,
																		 GParamSpec			*pspec)
{
  GemTranslucentWindowPriv *priv;
	gdouble double_val;
	gboolean bool_val;
	
  priv = GEM_TRANSLUCENT_WINDOW_GET_PRIVATE(object);

  switch(prop_id)
  {
  	case PROP_OPACITY:
  		double_val = g_value_get_double(value);
  		if (priv->opacity != double_val)
  		{
				priv->opacity = double_val;
				if (GTK_WIDGET_REALIZED(GTK_WIDGET(object)))
				{
					gtk_widget_queue_draw(GTK_WIDGET(object));
				}  			
  		}
  		break;
  		
   	case PROP_FAKE_TRANS:
  		bool_val = g_value_get_boolean(value);
  		if (priv->fake_trans != bool_val)
  		{
  			priv->fake_trans = bool_val;
  			
				if (GTK_WIDGET_REALIZED(GTK_WIDGET(object)))
				{
					gtk_widget_queue_draw(GTK_WIDGET(object));
				}  			
  		}
  		break;
  		
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
			break;
	}
	
  return;
}

static void
gem_translucent_window_get_property (GObject			*object,
																		 guint				 prop_id,
																		 GValue				*value,
																		 GParamSpec		*pspec)
{
  GemTranslucentWindowPriv *priv;

  priv = GEM_TRANSLUCENT_WINDOW_GET_PRIVATE(object);

  switch(prop_id)
  {
  	case PROP_OPACITY:
  		g_value_set_double(value, priv->opacity);
  		break;
  		
   	case PROP_FAKE_TRANS:
  		g_value_set_boolean(value, priv->fake_trans);
  		break;
  		
 		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
			break;
	}
  
  return;
}

static void
gem_translucent_window_init (GemTranslucentWindow *window)
{
	GemTranslucentWindowPriv *priv = GEM_TRANSLUCENT_WINDOW_GET_PRIVATE(window);
	Display *display = GDK_DISPLAY();
	int render_event, render_error;
	int comp_event, comp_error;
	int damage_event, damage_error;
	int fixes_event, fixes_error;
	
	if(XRenderQueryExtension(display, 
													 &render_event,
													 &render_error))
	{
		int major = 0, minor = 2;
		
		XRenderQueryVersion(display, 
												&major,
												&minor);
		g_print("%s(): XRender Version(%d.%d)\n",
						__FUNCTION__,
						major,
						minor);
	}
					
	if (XCompositeQueryExtension(display, 
															 &comp_event,
															 &comp_error))
	{
		int major = 0, minor = 2;
		
		XCompositeQueryVersion(display, 
													 &major,
													 &minor);
		g_print("%s(): XComposite Version(%d.%d)\n",
						__FUNCTION__,
						major,
						minor);
	}
	
	if (XDamageQueryExtension(display, 
														&damage_event,
														&damage_error))
	{
		int major = 0, minor = 2;
		
		XDamageQueryVersion(display, 
												&major,
												&minor);
		g_print("%s(): XDamage Version(%d.%d)\n",
						__FUNCTION__,
						major,
						minor);
		
		priv->xdamage_event_base = damage_event;
		
		gdk_x11_register_standard_event_type(gdk_display_get_default(),
                                         (priv->xdamage_event_base + XDamageNotify),
																					1);
		g_print("%s(): register XDamageEvent[base(%d), XDamageNotify(%d)]\n",
						__FUNCTION__,
						priv->xdamage_event_base,
						XDamageNotify);
	}
	
	if (XFixesQueryExtension(display, 
													 &fixes_event, 
													 &fixes_error))
	{
		int major = 0, minor = 2;
		
		XFixesQueryVersion(display, 
											 &major,
											 &minor);
		g_print("%s(): XFixes Version(%d.%d)\n",
						__FUNCTION__,
						major,
						minor);
	}
	
	gtk_widget_set_app_paintable(GTK_WIDGET(window),
															 TRUE);

	priv->window_entities = NULL;
	if (priv->window_entities == NULL)
	{
		Window *children = NULL;
    guint nchildren, i;
    Window root_w, parent_w;
    
		if (XQueryTree(display,
    		DefaultRootWindow(display), 
    		&root_w,
    		&parent_w,
    		&children,
    		&nchildren))
    {
    	g_print("%s(): root_w(%10p), parent_w(%10p), children(%10p), nchildren(%d)\n",
    					__FUNCTION__,
    					root_w,
    					parent_w,
    					children,
    					nchildren);
    	
    	for (i = 0; i < nchildren; i++)
    	{
				GemXWindowEntity *entity = NULL;
				
				entity = gem_xwindow_entity_new(children[i]);
				if (entity)
				{
					priv->window_entities = 
						g_list_append(priv->window_entities,
													(gpointer)entity);
				}
    	}
    	
			if (children)
			{
				XFree ((void *)children);
			}
    }
	}

	gdk_window_add_filter(NULL,
												(GdkFilterFunc)gem_translucent_window_xevent_filter,
												(gpointer)window);
	
	priv->fake_trans = FALSE;
	priv->opacity = 1.0;
	
	return;
}

GtkWidget *
gem_translucent_window_new (void)
{
	return gtk_widget_new(GEM_TYPE_TRANSLUCENT_WINDOW,
												"type",
												GTK_WINDOW_TOPLEVEL,
												NULL);
}

static GdkFilterReturn
gem_translucent_window_xevent_filter (GdkXEvent        		  *xevent,
																			GdkEvent          		*event,
																			GemTranslucentWindow	*window)
{
	GemTranslucentWindowPriv *priv = GEM_TRANSLUCENT_WINDOW_GET_PRIVATE(window);
	Display *display = GDK_DISPLAY();
	XEvent *e = xevent;

/*	g_print("%s(): event->type(%d)\n",
					__FUNCTION__,
					e->type);
*/
  if (e->type == (priv->xdamage_event_base + XDamageNotify))
  {
		XserverRegion parts;
		XDamageNotifyEvent *ev = xevent;
		XRectangle *rects;
		gint i, n_rect;

#if DETAIL_DEBUG
		g_print("%s(): priv->fake_trans(%s)\n",
						__FUNCTION__,
						(priv->fake_trans ? "true" : "false"));

    g_print("%s(): damage area: [%3d, %3d] - %3d x %3d\n",
    					__FUNCTION__,
    					ev->area.x,
    					ev->area.y,
    					ev->area.width,
    					ev->area.height);
#endif
				
    parts = XFixesCreateRegion(display, 
    													 0,
    													 0);

    XDamageSubtract(display,
    								ev->damage,
    								None,
    								parts);

		rects = XFixesFetchRegion(display,
															parts,
															&n_rect);

    XFixesDestroyRegion(display,
												parts);

    for (i = 0; !priv->fake_trans && i < n_rect; i++)
    {
    	gint tw_x, tw_y;
    	gint tw_w, tw_h;
    	
#if DETAIL_DEBUG    	
    	g_print("%s(): rect[%d]: [%3d, %3d] - %3d x %3d\n",
    					__FUNCTION__,
    					i,
    					rects[i].x,
    					rects[i].y,
    					rects[i].width,
    					rects[i].height);
#endif

			tw_w = GTK_WIDGET(window)->allocation.width;
			tw_h = GTK_WIDGET(window)->allocation.height;
			gdk_window_get_root_origin(GTK_WIDGET(window)->window,
																 &tw_x,
																 &tw_y);
	    	
			if (tw_x + tw_w > rects[i].x &&
					tw_x < rects[i].x + rects[i].width &&
					tw_y + tw_h > rects[i].y &&
					tw_y < rects[i].y + rects[i].height)

			{
        GdkRectangle rect;

        rect.x = MAX (rects[i].x, tw_x) - tw_x;
        rect.y = MAX (rects[i].y, tw_y) - tw_y;
        rect.width =
            MIN (rects[i].x + rects[i].width, tw_x + tw_w) -
            MAX (rects[i].x, tw_x);
        rect.height =
            MIN (rects[i].y + rects[i].height, tw_y + tw_h) -
            MAX (rects[i].y, tw_y);
    	
				gdk_window_invalidate_rect(GTK_WIDGET(window)->window,
																	 &rect,
																	 TRUE);
			}
		}
		
    XFree (rects);
  }

  return GDK_FILTER_CONTINUE;
}

/*
vi:ts=2:nowrap:ai:expandtab
*/
