/*	libgemwidget - Gtk+ Embedded Widget library
 * 
 *	gemxwindowentity.c - XWindow window entity object routines
 *
 *	Authors: YE Nan <nan.ye@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */
#include <stdio.h>
#include <stdlib.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/extensions/Xrender.h>
#include <X11/extensions/Xdamage.h>

#include <glib-object.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#include "gemxwindowentity.h"

#define	TRANSLUCENT		0xe0000000
#define	OPAQUE				0xffffffff

typedef struct _GemXWindowEntityPriv	GemXWindowEntityPriv;

struct _GemXWindowEntityPriv
{
	Window 							 win;
	XWindowAttributes		 attr;
	gint								 render_type;
	guint								 opacity;
	
	Damage							 damage;
	Picture							 picture;
};

#define GEM_XWINDOW_ENTITY_GET_PRIVATE(obj)	(G_TYPE_INSTANCE_GET_PRIVATE((obj), \
																						 GEM_TYPE_XWINDOW_ENTITY, \
																						 GemXWindowEntityPriv));

enum {
	PROP_GEM_XWINDOW_ENTITY_0,
	PROP_HANDLER,
	PROP_ATTR,
	PROP_RENDER_TYPE,
	PROP_OPACITY,
	PROP_PICTURE,
};

enum {
	GEM_XWINDOW_ENTITY_LAST_SIGNAL,
};

static void		gem_xwindow_entity_class_init			(GemXWindowEntityClass		*klass);
static void		gem_xwindow_entity_init						(GemXWindowEntity					*entity);
static void		gem_xwindow_entity_finalize				(GObject									*object);

static void		gem_xwindow_entity_set_property		(GObject									*object,
																								 guint										 prop_id,
																								 const GValue							*value,
																								 GParamSpec								*pspec);
static void		gem_xwindow_entity_get_property		(GObject									*object,
																								 guint										 prop_id,
																								 GValue										*value,
																								 GParamSpec								*pspec);

static void		_gem_xwindow_entity_determin_type	(GemXWindowEntity					*entity);

static GtkWindowClass *parent_class = NULL;
//static guint entity_signals[GEM_XWINDOW_ENTITY_LAST_SIGNAL];

GType
gem_xwindow_entity_get_type (void)
{
	static GType entity_type = 0;

  if (!entity_type)
	{
		static const GTypeInfo entity_info =
		{
			sizeof (GemXWindowEntityClass),
			NULL,		/* base_init */
			NULL,		/* base_finalize */
			(GClassInitFunc) gem_xwindow_entity_class_init,
			NULL,		/* class_finalize */
			NULL,		/* class_data */
			sizeof (GemXWindowEntity),
			0,			/* n_preallocs */
			(GInstanceInitFunc) gem_xwindow_entity_init,
			NULL,
		};
		
		entity_type = 
			g_type_register_static(G_TYPE_OBJECT,
														 "GemXWindowEntity",
														 &entity_info,
														 0);
	}
	
	return entity_type;
}

static void
gem_xwindow_entity_class_init (GemXWindowEntityClass * klass)
{
  GObjectClass *object_class;
  GParamSpec *handler_param;
  GParamSpec *attr_param;
  GParamSpec *render_type_param;
  GParamSpec *opacity_param;
  GParamSpec *picture_param;
 
	parent_class = g_type_class_peek_parent(klass);
  object_class = G_OBJECT_CLASS(klass);
	
	handler_param = 
		g_param_spec_uint("handler",
											"Handler",
											"XWindow Handler",
											0,
											UINT_MAX,
											None,
											G_PARAM_READABLE);
	
	attr_param = 
		g_param_spec_pointer("attribute",
										 		 "Attribute",
										 		 "XWindow Attribute",
										 		 G_PARAM_READABLE);

	render_type_param = 
		g_param_spec_uint("render-type",
											"Render Type",
											"XWindow Rendering Type",
											GEM_XWIN_RENDER_NOP,
											GEM_XWIN_RENDER_ARGB,
											GEM_XWIN_RENDER_NOP,
											G_PARAM_READABLE);
	
 	opacity_param = 
		g_param_spec_uint("opacity",
											"Opacity",
											"Window Opacity",
											0,
											UINT_MAX,
											OPAQUE,
											G_PARAM_READABLE);
	
	picture_param = 
		g_param_spec_ulong("picture",
											 "Picture",
											 "XRender Picture",
											 0,
											 ULONG_MAX,
											 None,
										 	 G_PARAM_READABLE);

  object_class->finalize			= gem_xwindow_entity_finalize;
	object_class->set_property	= gem_xwindow_entity_set_property;
	object_class->get_property	= gem_xwindow_entity_get_property;

	g_type_class_add_private(klass,
													 sizeof(GemXWindowEntityPriv));

	g_object_class_install_property(object_class, 
																	PROP_HANDLER,
																	handler_param);
																	
	g_object_class_install_property(object_class, 
																	PROP_ATTR,
																	attr_param);
																	
	g_object_class_install_property(object_class, 
																	PROP_RENDER_TYPE,
																	render_type_param);
																	
	g_object_class_install_property(object_class, 
																	PROP_OPACITY,
																	opacity_param);
																	
	g_object_class_install_property(object_class, 
																	PROP_PICTURE,
																	picture_param);
																	
	return;
}

static void
gem_xwindow_entity_finalize (GObject *object)
{
	GemXWindowEntityPriv *priv = GEM_XWINDOW_ENTITY_GET_PRIVATE(object);
	Display *display = GDK_DISPLAY();
	
	if (priv->damage)
	{
		XDamageDestroy(display, 
									 priv->damage);
		priv->damage = None;
	}
	
	if (priv->picture)
	{
		XRenderFreePicture(display, 
											 priv->picture);
    priv->picture = None;
	}
	
	G_OBJECT_CLASS(parent_class)->finalize(object);

	return;
}

static void
gem_xwindow_entity_set_property (GObject				*object,
																 guint					 prop_id,
																 const GValue		*value,
																 GParamSpec			*pspec)
{
  GemXWindowEntityPriv *priv;
	guint uint_val;
	
  priv = GEM_XWINDOW_ENTITY_GET_PRIVATE(object);

  switch(prop_id)
  {
  	case PROP_OPACITY:
  	{
  		uint_val = g_value_get_uint(value);
  		if (priv->opacity != uint_val)
  		{
  			priv->opacity = uint_val;
  			_gem_xwindow_entity_determin_type(GEM_XWINDOW_ENTITY(object));
  		}
  		break;
  	}
  	
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
			break;
	}
	
  return;
}

static Picture
_gem_xwindow_entity_create_picture (GemXWindowEntity *entity)
{
  GemXWindowEntityPriv *priv = GEM_XWINDOW_ENTITY_GET_PRIVATE(entity);
  Display *display = GDK_DISPLAY();
  Picture picture = None;
	XRenderPictFormat *format = NULL;
	XRenderPictureAttributes pa;

  format = XRenderFindVisualFormat(display, 
																	 priv->attr.visual);
																	 
	pa.subwindow_mode = IncludeInferiors;

	picture = 
		XRenderCreatePicture(display, 
												 priv->win,
												 format,
												 CPSubwindowMode,
												 &pa);
  
  return picture;
}

static void
gem_xwindow_entity_get_property (GObject			*object,
																 guint				 prop_id,
																 GValue				*value,
																 GParamSpec		*pspec)
{
  GemXWindowEntityPriv *priv;

  priv = GEM_XWINDOW_ENTITY_GET_PRIVATE(object);

  switch(prop_id)
  {
  	case PROP_HANDLER:
  		g_value_set_uint(value, priv->win);
  		break;
  	
  	case PROP_ATTR:
  		g_value_set_pointer(value, &(priv->attr));
  		break;
  	
  	case PROP_RENDER_TYPE:
  		g_value_set_uint(value, priv->render_type);
  		break;
  		
  	case PROP_OPACITY:
  		g_value_set_uint(value, priv->opacity);
  		break;  		
  	
  	case PROP_PICTURE:
  		if (priv->picture == None)
  		{
  			priv->picture =
  				_gem_xwindow_entity_create_picture(GEM_XWINDOW_ENTITY(object));
  		}
  		if (priv->picture)
  		{
  			g_value_set_ulong(value, priv->picture);
  		}
  		break;
  		
 		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
			break;
	}
  
  return;
}

static void
gem_xwindow_entity_init (GemXWindowEntity *entity)
{
	GemXWindowEntityPriv *priv = GEM_XWINDOW_ENTITY_GET_PRIVATE(entity);

	priv->win = None;
	priv->render_type = GEM_XWIN_RENDER_NOP;
	priv->opacity = OPAQUE;

	priv->damage = None;
	priv->picture = None;

	return;
}

static void
_gem_xwindow_entity_determin_type (GemXWindowEntity *entity)
{
	GemXWindowEntityPriv *priv = GEM_XWINDOW_ENTITY_GET_PRIVATE(entity);
	Display *display = GDK_DISPLAY();
	gint type = GEM_XWIN_RENDER_NOP;
	
	if (priv->attr.class == InputOnly)
	{
		type = GEM_XWIN_RENDER_NOP;
	}
	else
	{
		XRenderPictFormat *format = NULL;
		
		format = XRenderFindVisualFormat(display, 
																		 priv->attr.visual);
																		 
    if (format && 
    		format->type == PictTypeDirect && 
    		format->direct.alphaMask)
    {
			type = GEM_XWIN_RENDER_ARGB;
    }
    else if (priv->opacity != OPAQUE)
    {
			type = GEM_XWIN_RENDER_TRANS;
    }
    else
    {
			type = GEM_XWIN_RENDER_SOLID;
    }
	}

	priv->render_type = type;
	
	return;
}

GemXWindowEntity *
gem_xwindow_entity_new (Window win)
{
	GemXWindowEntityPriv *priv = NULL;
	GemXWindowEntity *entity = NULL;
	Display *display = GDK_DISPLAY();
	
	g_return_val_if_fail(win != None, NULL);
	
	entity = g_object_new(GEM_TYPE_XWINDOW_ENTITY,
												NULL);
	g_return_val_if_fail(entity, NULL);
	
	priv = GEM_XWINDOW_ENTITY_GET_PRIVATE(entity);
	
	priv->win = win;
	
	if (!XGetWindowAttributes(display, 
														priv->win,
														&(priv->attr)))
  {
  	goto cleanup;
  }
	
	priv->damage = XDamageCreate(display, 
															 priv->win,
															 XDamageReportNonEmpty);
															 
	_gem_xwindow_entity_determin_type(entity);
	
	g_print("%s(): entity(%10p), XID(%10p), render(%d), opacity(%d), [%4d, %4d] - %4d x %4d\n",
					__FUNCTION__,
					entity,
					priv->win,
					priv->render_type,
					priv->opacity,
					priv->attr.x,
					priv->attr.y,
					priv->attr.width,
					priv->attr.height);
					
	return entity;

cleanup:
	if (entity)
	{
		g_object_unref(entity);
	}
	
	return NULL;
}

/*
vi:ts=2:nowrap:ai:expandtab
*/
