/*	libgemwidget - Gtk+ Embedded Widget library
 *
 *	Authors: YE Nan <nan.ye@orange-ftgourp.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <ctype.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <string.h>
#include <stdlib.h>

#include <gtkdateedit.h>


static GtkHBoxClass * parent_class = NULL;

static void	gtk_date_edit_class_init		(GtkDateEditClass	*klass);
static void	gtk_date_edit_init					(GtkDateEdit			*window);
static void	gtk_date_edit_finalize			(GObject										*object);
static void	gtk_date_edit_destroy				(GtkObject									*object);

GType
gtk_date_edit_get_type (void)
{
	static GType date_edit_type = 0;

  if (!date_edit_type)
	{
		static const GTypeInfo date_edit_info =
		{
			sizeof (GtkDateEditClass),
			NULL,		/* base_init */
			NULL,		/* base_finalize */
			(GClassInitFunc) gtk_date_edit_class_init,
			NULL,		/* class_finalize */
			NULL,		/* class_data */
			sizeof (GtkDateEdit),
			0,			/* n_preallocs */
			(GInstanceInitFunc) gtk_date_edit_init,
		};

		date_edit_type = g_type_register_static (GTK_TYPE_HBOX,
																												"GtkDateEdit", 
																												&date_edit_info, 
																												0);
	}

	return date_edit_type;
}

static void
gtk_date_edit_class_init (GtkDateEditClass * klass)
{
  GObjectClass * object_class;
  GtkObjectClass * gtk_object_class;

	parent_class = g_type_class_peek_parent (klass);
	
  gtk_object_class = (GtkObjectClass *)klass;
  gtk_object_class->destroy = gtk_date_edit_destroy;

  object_class = (GObjectClass *)klass;
  object_class->finalize = gtk_date_edit_finalize;
 	  
	return;
}

static void	gtk_date_edit_destroy (GtkObject * object)
{
	GTK_OBJECT_CLASS(parent_class)->destroy(object);
  
	return;
}

static void
gtk_date_edit_finalize (GObject * object)
{
	G_OBJECT_CLASS(parent_class)->finalize(object);
	
	return;
}

void year_edit_insert_text_handler (GtkEntry    *entry,
                          const gchar *text,
                          gint         length,
                          gint        *position,
                          gpointer     data)
{
  GtkEditable *editable = GTK_EDITABLE(entry);
  int i, count=0;
	gchar *result = g_new (gchar, length);
		
	for (i=0; i < length; i++) {
    if (!isdigit(text[i]))
      continue;
    result[count++] = text[i];
  }
	
		if (count > 0) 
			{
				g_signal_handlers_block_by_func (G_OBJECT (editable),
																				 G_CALLBACK (year_edit_insert_text_handler),
																				 data);
					
				if (strlen(gtk_entry_get_text (GTK_ENTRY (entry))) <= 2 )
					gtk_editable_insert_text (editable, result, count, position);
				else if (strlen(gtk_entry_get_text (GTK_ENTRY (entry))) == 3)
					{
							gint year = atoi (gtk_entry_get_text (GTK_ENTRY (entry)));
									
							year = year * 10 + atoi (result);
									
							if (year <= 2037)
								gtk_editable_insert_text (editable, result, count, position);
							else
								{
								year = 2037;
																			
								sprintf (result,"%d",year);	
								gtk_entry_set_text (entry, result);
										
								}
									
							gtk_widget_child_focus (gtk_widget_get_parent (GTK_WIDGET (entry)), GTK_DIR_TAB_FORWARD);
							}							
										
					
				g_signal_handlers_unblock_by_func (G_OBJECT (editable),
																					 G_CALLBACK (year_edit_insert_text_handler),
																					 data);
			}
		
		g_signal_stop_emission_by_name (G_OBJECT (editable), "insert_text");
	
		g_free (result);
	
}

void mon_edit_insert_text_handler (GtkEntry    *entry,
                          const gchar *text,
                          gint         length,
                          gint        *position,
                          gpointer     data)
{
  GtkEditable *editable = GTK_EDITABLE(entry);
  int i, count=0;
	gchar *result = g_new (gchar, length);
	
	
	for (i=0; i < length; i++) {
    if (!isdigit(text[i]))
      continue;
    result[count++] = text[i];
  }
	
		if (count > 0) 
			{
				g_signal_handlers_block_by_func (G_OBJECT (editable),
																				 G_CALLBACK (mon_edit_insert_text_handler),
																				 data);
					
					
				if (strlen(gtk_entry_get_text (GTK_ENTRY (entry))) == 0)
					gtk_editable_insert_text (editable, result, count, position);
				else if (strlen(gtk_entry_get_text (GTK_ENTRY (entry))) == 1)
					{
						gint mon = atoi (gtk_entry_get_text (GTK_ENTRY (entry)));
										
						mon = mon * 10 + atoi (result);
										
						if (mon <= 12)
							gtk_editable_insert_text (editable, result, count, position);
						else
							{
								mon = 12;
																				
								sprintf (result,"%d", mon);	
								gtk_entry_set_text (entry, result);
											
								}
								
						gtk_widget_child_focus (gtk_widget_get_parent (GTK_WIDGET (entry)), GTK_DIR_TAB_FORWARD);
						}
													
					
				g_signal_handlers_unblock_by_func (G_OBJECT (editable),
																					 G_CALLBACK (mon_edit_insert_text_handler),
																					 data);
			}
		
		g_signal_stop_emission_by_name (G_OBJECT (editable), "insert_text");
	
		g_free (result);
	
}

void day_edit_insert_text_handler (GtkEntry    *entry,
                          const gchar *text,
                          gint         length,
                          gint        *position,
                          gpointer     data)
{
  GtkEditable *editable = GTK_EDITABLE(entry);
  int i, count=0;
	gchar *result = g_new (gchar, length);
	
	for (i=0; i < length; i++) {
    if (!isdigit(text[i]))
      continue;
    result[count++] = text[i];
  }
	
		if (count > 0) 
			{
				g_signal_handlers_block_by_func (G_OBJECT (editable),
																				 G_CALLBACK (day_edit_insert_text_handler),
																				 data);
						
				if (strlen(gtk_entry_get_text (GTK_ENTRY (entry))) == 0)
					gtk_editable_insert_text (editable, result, count, position);
				else if (strlen(gtk_entry_get_text (GTK_ENTRY (entry))) == 1)
					{
						gint date = atoi (gtk_entry_get_text (GTK_ENTRY (entry)));
										
						date = date * 10 + atoi (result);
										
						if (date <= 31)
							gtk_editable_insert_text (editable, result, count, position);
						else
							{
								date = 31;
																				
								sprintf (result,"%d",date);	
								gtk_entry_set_text (entry, result);
											
							}
											
						gtk_widget_child_focus (gtk_widget_get_parent (GTK_WIDGET (entry)), GTK_DIR_TAB_BACKWARD);
						gtk_widget_child_focus (gtk_widget_get_parent (GTK_WIDGET (entry)), GTK_DIR_TAB_BACKWARD);
				}	
									
				g_signal_handlers_unblock_by_func (G_OBJECT (editable),
																					 G_CALLBACK (day_edit_insert_text_handler),
																					 data);
			}
		
		g_signal_stop_emission_by_name (G_OBJECT (editable), "insert_text");
	
		g_free (result);
	
}
static gboolean year_key_press_handler (GtkWidget *widget, GdkEventKey *k, gpointer user_data)
{
	
	
	if (k->keyval == GDK_Right) 
    {
			
			gtk_widget_child_focus (gtk_widget_get_parent (widget), GTK_DIR_TAB_FORWARD);
			
			return TRUE;
		}
	else if (k->keyval == GDK_Left) 
    {
			
			gtk_widget_child_focus (gtk_widget_get_parent (widget), GTK_DIR_TAB_FORWARD);
			gtk_widget_child_focus (gtk_widget_get_parent (widget), GTK_DIR_TAB_FORWARD);
			
			return TRUE;
		}
	else
		return FALSE;
}

static gboolean mon_key_press_handler (GtkWidget *widget, GdkEventKey *k, gpointer user_data)
{
		
	if (k->keyval == GDK_Right) 
    {			
			gtk_widget_child_focus (gtk_widget_get_parent (widget), GTK_DIR_TAB_FORWARD);			
			return TRUE;
		}
	else if (k->keyval == GDK_Left) 
    {		
			gtk_widget_child_focus (gtk_widget_get_parent (widget), GTK_DIR_TAB_BACKWARD);			
			return TRUE;
		}
	else
		return FALSE;
}

static gboolean day_key_press_handler (GtkWidget *widget, GdkEventKey *k, gpointer user_data)
{
		
	if (k->keyval == GDK_Right) 
    {			
			gtk_widget_child_focus (gtk_widget_get_parent (widget), GTK_DIR_TAB_BACKWARD);	
			gtk_widget_child_focus (gtk_widget_get_parent (widget), GTK_DIR_TAB_BACKWARD);			
			return TRUE;
		}
	else if (k->keyval == GDK_Left) 
    {		
			gtk_widget_child_focus (gtk_widget_get_parent (widget), GTK_DIR_TAB_BACKWARD);			
			return TRUE;
		}
	else
		return FALSE;
}


void 				gtk_date_edit_set_widget_width			(GtkDateEdit 	*entry,
																								guint 					pad1w, 
																								guint 					entry1w,
																								guint 					entry2w,
																								guint 					entry3w,
																								guint						pad2w)
{
	
	gtk_widget_set_size_request (entry->pad1, pad1w, -1);
	gtk_widget_set_size_request (entry->entry1, entry1w, -1);
	gtk_widget_set_size_request (entry->entry2, entry2w, -1);
	gtk_widget_set_size_request (entry->entry3, entry3w, -1);
	gtk_widget_set_size_request (entry->pad2, pad2w, -1);
}

static gboolean entry_focus_out_cb (GtkWidget * widget, GdkEventFocus * event, 
																												gpointer user_data)
{
	
	gtk_editable_select_region(GTK_EDITABLE(widget), 0, 0);

	return FALSE;	
}

static void
gtk_date_edit_init (GtkDateEdit * date_edit)
{
		
	
	date_edit->pad1 = gtk_label_new ("");
	date_edit->entry1 = gtk_entry_new();
	date_edit->entry2 = gtk_entry_new ();
	date_edit->entry3 = gtk_entry_new ();
	date_edit->eventbox1 = gtk_event_box_new ();
	date_edit->eventbox2 = gtk_event_box_new ();
	date_edit->pad2 = gtk_label_new ("");
	
	g_signal_connect(G_OBJECT(date_edit->entry1), "insert_text",
		     G_CALLBACK(year_edit_insert_text_handler),
					NULL);
	g_signal_connect(G_OBJECT(date_edit->entry2), "insert_text",
		     G_CALLBACK(mon_edit_insert_text_handler),
		      NULL);		
	g_signal_connect(G_OBJECT(date_edit->entry3), "insert_text",
		     G_CALLBACK(day_edit_insert_text_handler),
		      NULL);		
	g_signal_connect(G_OBJECT(date_edit->entry1), "key_press_event",
		     G_CALLBACK(year_key_press_handler),
					NULL);
	g_signal_connect(G_OBJECT(date_edit->entry2), "key_press_event",
		     G_CALLBACK(mon_key_press_handler),
		      NULL);			
	g_signal_connect(G_OBJECT(date_edit->entry3), "key_press_event",
		     G_CALLBACK(day_key_press_handler),
		      NULL);							
					
	g_signal_connect(G_OBJECT(date_edit->entry1), "focus-out-event",
											G_CALLBACK(entry_focus_out_cb), NULL);
	g_signal_connect(G_OBJECT(date_edit->entry2), "focus-out-event",
											G_CALLBACK(entry_focus_out_cb), NULL);
	g_signal_connect(G_OBJECT(date_edit->entry3), "focus-out-event",
											G_CALLBACK(entry_focus_out_cb), NULL);				
											
	gtk_entry_set_has_frame(GTK_ENTRY(date_edit->entry1), FALSE);
	gtk_entry_set_has_frame(GTK_ENTRY(date_edit->entry2), FALSE);
	gtk_entry_set_has_frame(GTK_ENTRY(date_edit->entry3), FALSE);
	
	gtk_box_pack_start (GTK_BOX (date_edit), date_edit->pad1, FALSE ,FALSE, 0);
	gtk_box_pack_start (GTK_BOX (date_edit), date_edit->entry1, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (date_edit), date_edit->eventbox1, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (date_edit), date_edit->entry2, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (date_edit), date_edit->eventbox2, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (date_edit), date_edit->entry3, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (date_edit), date_edit->pad2, FALSE ,FALSE, 0);
	
	
//	GTK_WIDGET_SET_FLAGS (date_edit, GTK_CAN_FOCUS);
	
	date_edit->label1 = gtk_label_new ("-");
	gtk_container_add(GTK_CONTAINER (date_edit->eventbox1), date_edit->label1);
	date_edit->label2 = gtk_label_new ("-");
	gtk_container_add(GTK_CONTAINER (date_edit->eventbox2), date_edit->label2);
	
  GdkColor white;
	
	white.red = 65535;
  white.green = 65535;
  white.blue = 65535;
										
	gtk_widget_modify_bg (date_edit->eventbox1, GTK_STATE_NORMAL, &white);
	gtk_widget_modify_bg (date_edit->eventbox2, GTK_STATE_NORMAL, &white);
	gtk_entry_set_width_chars (GTK_ENTRY (date_edit->entry1), 4);
	gtk_entry_set_width_chars (GTK_ENTRY (date_edit->entry2), 2);
	gtk_entry_set_width_chars (GTK_ENTRY (date_edit->entry3), 2);	
	
	return;
}

guint		gtk_date_edit_get_year	(GtkDateEdit  *dateedit)
{
	guint year;
	
	year = atoi(gtk_entry_get_text(GTK_ENTRY(dateedit->entry1)));
	//printf ("%s:year=%d\n", __FUNCTION__, year);
	
	return year;
	
}

guint		gtk_date_edit_get_mon	(GtkDateEdit  *dateedit)
{
	guint mon;
	
	mon = atoi(gtk_entry_get_text(GTK_ENTRY(dateedit->entry2)));
	//printf ("%s:month=%d\n", __FUNCTION__, mon);
	
	return mon;
	
}

guint		gtk_date_edit_get_day	(GtkDateEdit  *dateedit)
{
	guint day;
	
	day = atoi(gtk_entry_get_text(GTK_ENTRY(dateedit->entry3)));
	//printf ("%s:day=%d\n", __FUNCTION__, day);
	
	return day;
	
}

void				gtk_date_edit_set_value							(GtkDateEdit  *dateedit,
																								guint						year,
																								guint						mon,
																								guint						day)
{
	gchar textbuf[5];
	
	sprintf (textbuf, "%d", year);
	gtk_entry_set_text (GTK_ENTRY (dateedit->entry1), textbuf);
	if (mon > 9)
		sprintf (textbuf, "%d", mon);
	else
		sprintf (textbuf, "0%d", mon);
	gtk_entry_set_text (GTK_ENTRY (dateedit->entry2), textbuf);
	if (day > 9)
		sprintf (textbuf, "%d", day);
	else
		sprintf (textbuf, "0%d", day);
	gtk_entry_set_text (GTK_ENTRY (dateedit->entry3), textbuf);
	
	
}

GtkWidget *
gtk_date_edit_new (void)
{
  return gtk_widget_new (GTK_TYPE_DATE_EDIT, NULL);
}
