/*	libgemwidget - Gtk+ Embedded Widget library
 * 
 *	freq-update-window.c - Frequently self-update window
 *
 *	Authors: YE Nan <nan.ye@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <gem.h>

#define	TIME_DELTA		1000

static gboolean 
update_label (GtkLabel	*label)
{
	time_t current_time;
	struct tm *ltime;
	gchar *time_str = NULL;
	
	current_time = time(NULL);
	ltime = localtime(&current_time);
	
/*	g_print("%s(): %02d:%02d:%02d\n",
					__FUNCTION__,
					ltime->tm_hour,
					ltime->tm_min,
					ltime->tm_sec);
*/	
	time_str = 
		g_strdup_printf("%02d:%02d:%02d",
										ltime->tm_hour,
										ltime->tm_min,
										ltime->tm_sec);

	gtk_label_set_text(label, time_str);
	g_free(time_str);
											
	return TRUE;
}

int
main (int		 argc,
			char	*argv[])
{
	GtkWidget *main_window = NULL;
	GtkWidget *vbox = NULL;
	GtkWidget *label = NULL;
	GtkWidget *button = NULL;
	
	gtk_init(&argc, &argv);
	
	main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	g_signal_connect(G_OBJECT(main_window),
									 "destroy",
									 G_CALLBACK(gtk_main_quit),
									 NULL);
										
	vbox = gtk_vbox_new(FALSE, 0);
	
	gtk_box_pack_start(GTK_BOX(vbox),
										 gtk_combo_box_new(),
										 FALSE,
										 FALSE,
										 0);
										 
	label = gtk_label_new("00:00:00");
	g_timeout_add(3 * TIME_DELTA,
								(GSourceFunc)update_label,
								(gpointer)label);
								
	gtk_box_pack_start(GTK_BOX(vbox),
										 label,
										 TRUE,
										 TRUE,
										 0);
										 
	gtk_box_pack_start(GTK_BOX(vbox),
										 gtk_entry_new(),
										 FALSE,
										 FALSE,
										 0);

	button = gtk_button_new_with_label("Close");
	g_signal_connect_swapped(G_OBJECT(button),
													 "clicked",
													 G_CALLBACK(gtk_widget_destroy),
													 (gpointer)main_window);
													 
	gtk_box_pack_end(GTK_BOX(vbox),
									 gtk_button_new_with_label("Button"),
									 FALSE,
									 FALSE,
									 0);
	gtk_container_add(GTK_CONTAINER(main_window),
										vbox);
	
	gtk_widget_show_all(main_window);
	
	gtk_main();
	
	return EXIT_SUCCESS;
}
