/*	libgemwidget - Gtk+ Embedded Widget library
 * 
 *	test-transwindow.c - Test for GemTranslucentWindow object
 *
 *	Authors: YE Nan <nan.ye@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */
#include <stdio.h>
#include <stdlib.h>

#include <gem.h>

static void
adjustment_changed (GtkAdjustment		*adjustment,
                    GtkWidget				*window)
{
	gdouble opacity;
	
	opacity = gtk_adjustment_get_value(adjustment);
/*	g_print("%s(): opacity(%f)\n",
					__FUNCTION__,
					opacity);
*/	
	g_object_set(G_OBJECT(window),
							 "opacity",
							 opacity,
							 NULL);
						 
	return;
}

static void
check_clicked (GtkButton	*button,
               GtkWidget	*window)
{
	gboolean toggled;
	
	g_object_get(G_OBJECT(button),
							 "active",
							 &toggled,
							 NULL);

	g_object_set(G_OBJECT(window),
							 "fake-translucent",
							 toggled,
							 NULL);
	
	return;
}

int
main (int		 argc,
			char	*argv[])
{
	GtkWidget *main_window = NULL;
	GtkWidget *vbox = NULL;
	GtkWidget *label = NULL;
	GtkWidget *hscale = NULL;
	GtkWidget *check = NULL;
	GtkAdjustment	*adjustment;
	
	gtk_init(&argc, &argv);
	
	main_window =	
		gtk_widget_new(GEM_TYPE_TRANSLUCENT_WINDOW,
//									 "type",
//									 GTK_WINDOW_POPUP,
//									 "type-hint",
//									 GDK_WINDOW_TYPE_HINT_DOCK,
									 NULL);

	gtk_window_move(GTK_WINDOW(main_window),
									0, 100);
	gtk_window_resize(GTK_WINDOW(main_window),
										240, 150);
	gtk_window_set_decorated(GTK_WINDOW(main_window),
													 FALSE);
													 
	vbox = gtk_vbox_new(FALSE, 0);
	
	label = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(label),
											 "A <span foreground='red'>Translucent</span> Window");
	gtk_box_pack_start(GTK_BOX(vbox),
										 label,
										 TRUE,
										 TRUE,
										 0);
	
	adjustment = 
		GTK_ADJUSTMENT(gtk_adjustment_new(0.5,
																			0.0,
																			1.0,
																			0.05,
																			0.1,
																			0.0));
	gtk_adjustment_set_value(GTK_ADJUSTMENT(adjustment),
													 1.0);
													 													 
	hscale = gtk_hscale_new_with_range(0.0,
																		 1.0,
																		 0.05);
	g_object_set(G_OBJECT(hscale),
							 "adjustment",
							 adjustment,
							 NULL);	
	g_signal_connect(G_OBJECT(adjustment),
									 "value-changed",
									 G_CALLBACK(adjustment_changed),
									 (gpointer)main_window);
	gtk_box_pack_start(GTK_BOX(vbox),
										 hscale,
										 FALSE,
										 FALSE,
										 0);

	check = gtk_check_button_new_with_label("Fake Translucent");
/*	g_object_set(G_OBJECT(check),
							 "active",
							 TRUE,
							 NULL);	
*/
	g_signal_connect(G_OBJECT(check),
									 "clicked",
									 G_CALLBACK(check_clicked),
									 (gpointer)main_window);
	gtk_box_pack_start(GTK_BOX(vbox),
										 check,
										 FALSE,
										 FALSE,
										 0);
																		 
	gtk_box_pack_start(GTK_BOX(vbox),
										 gtk_button_new_with_label("Button"),
										 FALSE,
										 FALSE,
										 0);
	gtk_container_add(GTK_CONTAINER(main_window),
										vbox);
	
	gtk_widget_show_all(main_window);
	
	gtk_main();
	
	return EXIT_SUCCESS;
}
